/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Produktlinie C14

  Description:  definition file for io driver

  -------------------------------------------------------------------------

                $RCSfile: C14Def.h,v $
                
                $Author: A.Suess $
                
                $Revision: 1.2 $  $Date: 2004/09/23 13:18:11 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/03/30 -as:   set definitions for C14

****************************************************************************/

#ifndef _C14DEF_H_
#define _C14DEF_H_

#include <xc161.h>

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define PCB_CPU_VERSION     4112
#define PCB_CPU_REVISION    0
//#define HWID_CPU            0x00

#define PCB_IO_VERSION      4103
#define PCB_IO_REVISION     1
#define HWID_IO_PROTOTYPE   0x07
#define HWID_IO_NULLSERIE   0x06

// ------------- Memory mapped IOs ------------------------------------------
// The following defines must fit to the set in startup-file.
// FLASH: 0x00:0000 - 0x1F:FFFF (2MiB)
// CAN:   0x20:0000 - 0x20:07FF (TWIN-CAN-Modul)
// ETH:   0x20:2000 - 0x20:2FFF (4kiB Ethernet-Controller)
// nvSRAM:0x30:0000 - 0x30:FFFF (64kiB, externer Adressdekoder A19=0)
// XUART: 0x38:0000 - 0x38:0FFF (4kiB, externer Adressdekoder A19=1)
// RAM1:  0x40:0000 - 0x4F:FFFF (1MiB)
// RAM2:  0x50:0000 - 0x5F:FFFF (1MiB)
// FLASH: 0x60:0000 - 0x7F:FFFF (2MiB-Spiegel)
// FLASH: 0xC0:0000 - 0xC1:FFFF (128kiB on-chip Flash)
// PMA:   0xC0:0000 - 0xFF:FFFF (4MiB on-chip Programm memory area)

#define BASEADDR_CS_ETH     0x202000 // Chip Select of Ethernet controller
#define BASEADDR_CS_NVSRAM  0x300000 // Chip Select of nvSRAM (A19 = 0)
#define BASEADDR_CS_XUART   0x380000 // Chip Select of XUART  (A19 = 1)


// ------------- Dedicated Pins ----------------------------------------------
// SPI-Interface
sbit    SPI_A0        = P2^8;       // SPI address line 0
sbit    SPI_A1        = P2^9;       // SPI address line 1
sbit    SPI_A2        = P2^10;      // SPI address line 2
sbit    SPI_A3        = P2^11;      // SPI address line 3
sbit    SPI_MRST      = P3^8;       // SPI master receive line
sbit    SPI_MTSR      = P3^9;       // SPI master transmit line
sbit    SPI_CLK       = P3^13;      // SPI clock line
sbit    SPI_MR        = P6^5;       // SPI master reset line
sbit    SPI_PL        = P6^6;       // SPI strobe/parallel load line
sbit    SPI_OE        = P6^7;       // SPI output enable line

// RTC-Interface
sbit    RTC_INT       = P2^12;      // RTC interrupt line
sbit    RTC_SDA       = P9^4;       // RTC data line
sbit    RTC_SCL       = P9^5;       // RTC clock line (IC)
sbit    RTC_CLK       = P5^12;      // RTC clock line (1Hz-clock)

// Ethernet-Interface
sbit    ETH_INT       = P2^13;      // Ethernet interrupt line
sbit    ETH_CLK       = P3^15;      // Ethernet oscillator clock line

// XUART-Interface
sbit    XUART_INT     = P2^14;      // XUART interrupt line

// ASC0/1-Interface
sbit    ASC0_TXD      = P3^10;      // ASC0 transmitter line
sbit    ASC0_RXD      = P3^11;      // ASC0 receiver line
sbit    ASC1_TXD      = P3^0;       // ASC1 transmitter line
sbit    ASC1_RXD      = P3^1;       // ASC1 receiver line

// Misc. Pins
sbit    CPU_PF        = P2^15;      // Power Fail Erkennung
sbit    CPU_TRG_WDT   = P3^2;       // Trigger Watchdog
sbit    CPU_ERR_LED   = P3^3;       // Error LED
sbit    CPU_RUN_LED   = P3^4;       // Run LED
sbit    CPU_SDC_DET   = P5^4;       // SD-Card detect line
sbit    CPU_BAT_LOW   = P5^5;       // Battery low line
sbit    CPU_SW_MRES   = P5^6;       // switch MRES line
sbit    CPU_SW_RUN    = P5^7;       // switch RUN line

// Process peripherals
sbit    CPU_CNT0      = P3^7;       // Counter 0 line
sbit    CPU_CNT1      = P3^6;       // Counter 1 line
sbit    CPU_CNT2      = P3^5;       // Counter 2 line
sbit    CPU_CNT3      = P5^13;      // Counter 3 line
sbit    CPU_CNT4      = P5^14;      // Counter 4 line
sbit    CPU_CNT5      = P5^15;      // Counter 5 line
sbit    CPU_UDC0      = P5^15;      // Up/Down control Counter 0
sbit    CPU_UDC2      = P5^14;      // Up/Down control Counter 2
sbit    CPU_PWM0      = P9^0;       // PWM 0 line
sbit    CPU_PWM1      = P9^1;       // PWM 1 line
sbit    CPU_PWM2      = P9^2;       // PWM 2 line
sbit    CPU_PWM3      = P9^3;       // PWM 3 line


// ------------- SPI device addresses ----------------------------------------------
// Bit Mask for SPI-address decoder
// The SPI address lines SPI_A0-3 are connected with address decoder.
#define SPI_CS_LED_DO   0x00    // Status-LEDs for digital outputs
#define SPI_CS_LED_DI   0x01    // Status-LEDs for digital inputs
#define SPI_CS_CODE     0x02    // DIP-Switch, HEX-Switch, configuration 
#define SPI_CS_EEPR1    0x03    // EEPROM1 CPU-board
#define SPI_CS_EEPR2    0x04    // EEPROM2 CPU-board
#define SPI_CS_SD       0x05    // SD-Card Connector
#define SPI_CS_DOUT_LED 0x06    // Status-LED and latch of digital outputs parallel activ
#define SPI_CS_DIN_LED  0x07    // Status-LED and register digital inputs parallel activ
#define SPI_CS_IO0      0x08    // Latch digital outputs
#define SPI_CS_IO1      0x09    // Register digital inputs
#define SPI_CS_HWCFG_IO 0x0A    // Register configuration IO-board
#define SPI_CS_IO3      0x0B    // Register IO3
#define SPI_CS_IO4      0x0C    // Register IO4
#define SPI_CS_IO5      0x0D    // Register IO5
#define SPI_CS_IO6      0x0E    // Register IO6
#define SPI_CS_INACTIV  0x0F    // no SPI device selected

#define SPI_SELECT(A)   (_bfld_(P2, 0x0F00, (A<<8)))

#define LED_ON          0
#define LED_OFF         1

// SPI baudrate constants
#define SPI_BR2M        0x0009
#define SPI_BR1M        0x0013
#define SPI_BR100k      0x00C7

//---------------------------------------------------------------------------

#endif
