/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      COMBImodul-C14

  Description:  source file for io driver api function 

  -------------------------------------------------------------------------

                $RCSfile: C14Drv.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.8 $  $Date: 2005/01/26 12:19:41 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/03/29 -as:   start of the implementation, version 1.00

  2004/11/04 -as:   function C14ResetTarget added

  2005/01/10 -as:   added function to register callback function to 
                    set an critical section for SPI ressources

****************************************************************************/

#include <intrins.h>
#include "generic/global.h"
#include "generic/gmacros.h"

#include "../include/C14drv.h"
#include "../include/C14def.h"



/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------
//#define WATCHDOG_ENABLED

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

bit bitPWMPortValueOne_g;
bit bitPWMPortValueZero_g;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 io driver>                                     */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14Initialize
//
// Description:  This function initialize all portpins for the C14. The 
//               function must called first befor calling other driver.
//
//
// Parameters:   none
//
// 
// Returns:      version of the driver
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

WORD PUBLIC C14Initialize (void)
{
DWORD dwDin;
DWORD dwDout;
BYTE  bHwCfgIo;

    ///  -----------------------------------------------------------------------
    ///  General Port Settings:
    ///  -----------------------------------------------------------------------
    PICON          =  0x0046;      // select input threshold
                                   // P2H  = CMOS input
                                   // P3L  = CMOS input
                                   // P3H  = TTL input
                                   // P4L  = TTL input
                                   // P6L  = TTL input
                                   // P7L  = CMOS input
                                   // P9L  = TTL input
                                   // P20L = TTL input
                                   // P20H = TTL input

    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P0H:
    ///  -----------------------------------------------------------------------
    ///  P0H.0 is used as alternate output for the Port Pin (AD8)
    ///  P0H.1 is used as alternate output for the Port Pin (AD9)
    ///  P0H.2 is used as alternate output for the Port Pin (AD10)
    ///  P0H.3 is used as alternate output for the Port Pin (AD11)
    ///  P0H.4 is used as alternate output for the Port Pin (AD12)
    ///  P0H.5 is used as alternate output for the Port Pin (AD13)
    ///  P0H.6 is used as alternate output for the Port Pin (AD14)
    ///  P0H.7 is used as alternate output for the Port Pin (AD15)
    
    ///  P0H.0 - P0H.3 output driver characteristic: strong driver
    ///  P0H.4 - P0H.7 output driver characteristic: strong driver
    
    ///  P0H.0 - P0H.3 output edge characteristic: soft edge mode
    ///  P0H.4 - P0H.7 output edge characteristic: soft edge mode
    
    P0H            =  0x0000;      // load data register
    DP0H           =  0x0000;      // load direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P0L:
    ///  -----------------------------------------------------------------------
    ///  P0L.0 is used as alternate output for the Port Pin (AD0)
    ///  P0L.1 is used as alternate output for the Port Pin (AD1)
    ///  P0L.2 is used as alternate output for the Port Pin (AD2)
    ///  P0L.3 is used as alternate output for the Port Pin (AD3)
    ///  P0L.4 is used as alternate output for the Port Pin (AD4)
    ///  P0L.5 is used as alternate output for the Port Pin (AD5)
    ///  P0L.6 is used as alternate output for the Port Pin (AD6)
    ///  P0L.7 is used as alternate output for the Port Pin (AD7)
    
    ///  P0L.0 - P0L.3 output driver characteristic: strong driver
    ///  P0L.4 - P0L.7 output driver characteristic: strong driver
    
    ///  P0L.0 - P0L.3 output edge characteristic: soft edge mode
    ///  P0L.4 - P0L.7 output edge characteristic: soft edge mode
    
    P0L            =  0x0000;      // load data register
    DP0L           =  0x0000;      // load direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P1H:
    ///  -----------------------------------------------------------------------
    ///  P1H.0 is used as alternate output for the Port Pin (A8)
    ///  P1H.1 is used as alternate output for the Port Pin (A9)
    ///  P1H.2 is used as alternate output for the Port Pin (A10)
    ///  P1H.3 is used as alternate output for the Port Pin (A11)
    ///  P1H.4 is used as alternate output for the Port Pin (A12)
    ///  P1H.5 is used as alternate output for the Port Pin (A13)
    ///  P1H.6 is used as alternate output for the Port Pin (A14)
    ///  P1H.7 is used as alternate output for the Port Pin (A15)
    
    ///  P1H.0 - P1H.3 output driver characteristic: strong driver
    ///  P1H.4 - P1H.7 output driver characteristic: strong driver
    
    ///  P1H.0 - P1H.3 output edge characteristic: soft edge mode
    ///  P1H.4 - P1H.7 output edge characteristic: soft edge mode
    
    P1H            =  0x0000;      // load data register
    ALTSEL0P1H     =  0x0000;      // load alternate output function select 
                                   // register 0
    DP1H           =  0x0000;      // load direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P1L:
    ///  -----------------------------------------------------------------------
    ///  P1L.0 is used as alternate output for the Port Pin (A0)
    ///  P1L.1 is used as alternate output for the Port Pin (A1)
    ///  P1L.2 is used as alternate output for the Port Pin (A2)
    ///  P1L.3 is used as alternate output for the Port Pin (A3)
    ///  P1L.4 is used as alternate output for the Port Pin (A4)
    ///  P1L.5 is used as alternate output for the Port Pin (A5)
    ///  P1L.6 is used as alternate output for the Port Pin (A6)
    ///  P1L.7 is used as alternate output for the Port Pin (A7)
    
    ///  P1L.0 - P1L.3 output driver characteristic: strong driver
    ///  P1L.4 - P1L.7 output driver characteristic: strong driver
    
    ///  P1L.0 - P1L.3 output edge characteristic: soft edge mode
    ///  P1L.4 - P1L.7 output edge characteristic: soft edge mode
    
    P1L            =  0x0000;      // load data register
    ALTSEL0P1L     =  0x0000;      // load alternate output function select 
                                 // register 0
    DP1L           =  0x0000;      // load direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P2:
    ///  -----------------------------------------------------------------------
    ///  P2.8 is used as general output:    SPI_A0  (push/pull, pin status is high)
    ///  P2.9 is used as general output:    SPI_A1  (push/pull, pin status is high)
    ///  P2.10 is used as general output:   SPI_A2  (push/pull, pin status is high)
    ///  P2.11 is used as general output:   SPI_A3  (push/pull, pin status is high)
    ///  P2.12 is used as general input:    RTC Interrupt line 
    ///  P2.13 is used as general input:    Ethernet Interrupt line
    ///  P2.14 is used as general input:    UART Interrupt line
    ///  P2.15 is used as general input:    PF Interrupt line
    
    ///  P2.8 - P2.15 threshold type: TTL input

    ///  P2.8-11 Addresslines for selection of SPI-IC's
    ///  default-value after reset is high (no IC enabled).
    
    ODP2           =  0x0000;      // open-drain register: push/pull outputs
    P2             =  0x0F00;      // data register: P2.8-11 high
    ALTSEL0P2      =  0x0000;      // alternate output function select: -
    POCON2         =  0x0000;      // output control register: strong driver, sharp edge
    DP2            =  0x0F00;      // direction register: P2.8-11 outputs
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P3:
    ///  -----------------------------------------------------------------------
    ///  P3.0 is used as alternate output:  ASC1_TxD (push/pull, pin status is high)
    ///  P3.1 is used as alternate input:   ASC1_RxD
    ///  P3.2 is used as general output:    Watchdog Trigger (output = Watchdog aktiv)
    ///  P3.3 is used as general output:    ERROR-LED (push/pull, pin status is high)
    ///  P3.4 is used as general output:    RUN-LED (push/pull, pin status is high)
    ///  P3.5 is used as general input:     Counter 2
    ///  P3.6 is used as general input:     Counter 1
    ///  P3.7 is used as general input:     Counter 0
    ///  P3.8 is used as alternate input:   SSC0_MRST
    ///  P3.9 is used as alternate output:  SSC0_MTSR (push/pull, pin status is high)
    ///  P3.10 is used as alternate output: ASC0_TxD (push/pull, pin status is high) 
    ///  P3.11 is used as alternate input:  ASC0_RxD
    ///  P3.13 is used as alternate output: SSC0_CLK (push/pull, pin status must set high 
    ///                                     for master mode !!)
    ///  P3.15 is used as alternate output: FOUT
    
    ///  P3.0 - P3.7  threshold type: TTL input
    ///  P3.8 - P3.15 threshold type: TTL input
    
    ///  P3.0 - P3.3 output driver characteristic: strong driver
    ///  P3.4 - P3.7 output driver characteristic: strong driver
    ///  P3.8 - P3.11 output driver characteristic: strong driver
    ///  P3.12 - P3.15 output driver characteristic: strong driver
    
    ///  P3.0 - P3.3 output edge characteristic: soft edge mode
    ///  P3.4 - P3.7 output edge characteristic: soft edge mode
    ///  P3.8 - P3.11 output edge characteristic: soft edge mode
    ///  P3.12 - P3.15 output edge characteristic: soft edge mode

    ODP3           =  0x0000;      // open-drain register: push/pull
    P3             =  0x2619;      // data register
    ALTSEL0P3      =  0x2701;      // alternate output function select register 0
    ALTSEL1P3      =  0x0000;      // alternate output function select register 1
    POCON3         =  0x2222;      // output control register: strong driver, soft edge
    DP3            =  0xA619;      // direction register (Watchdog disabled)

    #ifdef WATCHDOG_ENABLED
       _nop_();
       DP3         |= 0x0004;      // enable output driver = Watchdog enable
    #endif

    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P4:
    ///  -----------------------------------------------------------------------
    ///  P4.0 is used as alternate input for the Port Pin (A16)
    ///  P4.1 is used as alternate input for the Port Pin (A17)
    ///  P4.2 is used as alternate input for the Port Pin (A18)
    ///  P4.3 is used as alternate input for the Port Pin (A19)
    ///  P4.4 is used as alternate input for the Port Pin (A20)
    ///  P4.5 is used as alternate input for the Port Pin (A21)
    ///  P4.6 is used as alternate input for the Port Pin (A22)
    ///  P4.7 is used as alternate input for the Port Pin (A23)
    
    ///  P4.0 - P4.7 threshold type: TTL input
    
    ///  P4.0 - P4.3 output driver characteristic: strong driver
    ///  P4.4 - P4.7 output driver characteristic: strong driver
    
    ///  P4.0 - P4.3 output edge characteristic: soft edge mode
    ///  P4.4 - P4.7 output edge characteristic: soft edge mode
    
    ODP4           =  0x0000;      // open-drain register
    P4             =  0x0000;      // data register
    ALTSEL0P4      =  0x0000;      // alternate output function select register 0
    ALTSEL1P4      =  0x0000;      // alternate output function select register 1
    DP4            =  0x0000;      // direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of the used ADC Port Pins:
    ///  -----------------------------------------------------------------------
    ///  - P5.0 is used for Analog Input Pin (AN0)
    ///  - P5.1 is used for Analog Input Pin (AN1)
    ///  - P5.2 is used for Analog Input Pin (AN2)
    ///  - P5.3 is used for Analog Input Pin (AN3)
    
    ///  - digital input stage is disconnected from port line P5.0
    ///  - digital input stage is disconnected from port line P5.1
    ///  - digital input stage is disconnected from port line P5.2
    ///  - digital input stage is disconnected from port line P5.3
    
    P5DIDIS        =  0x000F;      // load Port 5 digital input disable register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P6:
    ///  -----------------------------------------------------------------------
    ///  P6.0 is used as alternate input for the Chip select (CS0_n)
    ///  P6.1 is used as alternate input for the Chip select (CS1_n)
    ///  P6.2 is used as alternate input for the Chip select (CS2_n)
    ///  P6.3 is used as alternate input for the Chip select (CS3_n)
    ///  P6.4 is used as alternate input for the Chip select (CS4_n)
    ///  P6.5 is used as general output: push/pull, pin status is high
    ///  P6.6 is used as general output: push/pull, pin status is high
    ///  P6.7 is used as general output: push/pull, pin status is high

    ///  P6.0 - P6.7 threshold type: TTL input
    
    ///  P6.0 - P6.3 output driver characteristic: strong driver
    ///  P6.4 - P6.7 output driver characteristic: strong driver
    
    ///  P6.0 - P6.3 output edge characteristic: soft edge mode
    ///  P6.4 - P6.7 output edge characteristic: soft edge mode
    
    ODP6           =  0x0000;      // open-drain register
    P6             =  0x00E0;      // data register
    ALTSEL0P6      =  0x0000;      // alternate output function select register 0
    DP6            =  0x00E0;      // direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P7:
    ///  -----------------------------------------------------------------------
    ///  P7.4 is used as alternate input for the CAN Interface Input (RXDCB)
    ///  P7.5 is used as alternate output for the CAN Interface Output (TXDCB)
    ///  - push/pull output is selected
    ///  P7.6 is used as alternate input for the CAN Interface Input (RXDCA)
    ///  P7.7 is used as alternate output for the CAN Interface Output (TXDCA)
    ///  - push/pull output is selected
    
    ///  P7.0 - P7.7 threshold type: TTL input
    
    ///  P7.0 - P7.3 output driver characteristic: strong driver
    ///  P7.4 - P7.7 output driver characteristic: strong driver
    
    ///  P7.0 - P7.3 output edge characteristic: soft edge mode
    ///  P7.4 - P7.7 output edge characteristic: soft edge mode
    
    ODP7           =  0x0000;      // open-drain register
    P7             =  0x0000;      // data register
    ALTSEL0P7      =  0x0000;      // alternate output function select register 0
    ALTSEL1P7      =  0x0000;      // alternate output function select register 1
    POCON7         =  0x0020;      // output control register: strong driver, soft edge
    DP7            =  0x0000;      // direction register
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P9:
    ///  -----------------------------------------------------------------------
    ///  P9.0 is used as alternate output: push/pull, pin status is low
    ///  P9.1 is used as alternate output: push/pull, pin status is low
    ///  P9.2 is used as alternate output: push/pull, pin status is low
    ///  P9.3 is used as alternate output: push/pull, pin status is low
    ///  P9.4 is used as general input:  switch to output into IC-driver
    ///  P9.5 is used as gereral input:  switch to output into IC-driver

    ///  P9.0 - P9.3 used as compare outputs (PWM)
    
    ///  P9.0 - P9.7 threshold type: TTL input
    
    ///  P9.0 - P9.3 output driver characteristic: strong driver
    ///  P9.4 - P9.7 output driver characteristic: strong driver
    
    ///  P9.0 - P9.3 output edge characteristic: soft edge mode
    ///  P9.4 - P9.7 output edge characteristic: soft edge mode
    
    ODP9           =  0x0000;      // open-drain register
    P9             =  0x0000;      // data register
    ALTSEL0P9      =  0x0000;      // alternate output function select register 0
    ALTSEL1P9      =  0x0000;      // alternate output function select register 1
    POCON9         =  0x0022;      // output control register: strong driver, soft edge
    DP9            =  0x000F;      // set direction register
    
    
    
    ///  -----------------------------------------------------------------------
    ///  Configuration of Port P20:
    ///  -----------------------------------------------------------------------
    ///  P20.2 is used as alternate input for the bus termination input signal 
    ///  (#READY)
    ///  P20.4 is used as alternate output for the address latch enable signal 
    ///  (ALE)
    
    ///  P20.0 - P20.7 threshold type: TTL input
    ///  P20.8 - P20.15 threshold type: TTL input
    
    ///  P20.0 - P20.3 output driver characteristic: strong driver
    ///  P20.4 - P20.7 output driver characteristic: strong driver
    ///  P20.12 - P20.15 output driver characteristic: strong driver
    
    ///  P20.0 - P20.3 output edge characteristic: soft edge mode
    ///  P20.4 - P20.7 output edge characteristic: soft edge mode
    ///  P20.12 - P20.15 output edge characteristic: sharp edge mode
    
    P20            =  0x0000;      // load data register
    DP20           =  0x0000;      // load direction register

    ///  -----------------------------------------------------------------------
    ///  Initialization of SPI Interface
    ///  -----------------------------------------------------------------------
    SPI_MR = 0;
    _nop_();
    SPI_MR = 1;

    C14SpiInit();

    // Before a SPI transfer is started the first time the 
    // Callback function must set to an defined value.
    C14SpiRegisterCbCriticalSection(NULL);

    ///  -----------------------------------------------------------------------
    ///  Initialization Bit-values for controlling PWM-output's
    ///  -----------------------------------------------------------------------
    ///  The active level of pwm-output is low activ on prototypes.
    ///  Since first series the active level is high
    ///  In a global variable the level will be defined depence on the HW-sign 
    ///  of the IO-PCB.

    bitPWMPortValueOne_g  = 1;
    bitPWMPortValueZero_g = 0;

    bHwCfgIo = C14GetIoHWCfg ();
    if ( (bHwCfgIo & 0x07) == HWID_IO_PROTOTYPE)
    {
       bitPWMPortValueOne_g  = 0;
       bitPWMPortValueZero_g = 1;
    }



    ///  -----------------------------------------------------------------------
    ///  reset all outputs
    ///  -----------------------------------------------------------------------
    C14GetInGroup ((BYTE FAR*)&dwDin, 4);  // RESET Status LEDs for digital Inputs

    dwDout = 0x0L;

    C14SetOutGroup((BYTE FAR *)&dwDout, 3);

    SPI_OE = 0;        // Enable all SPI latch driver

    ///  -----------------------------------------------------------------------
    ///  Initialization of counter inputs
    ///  -----------------------------------------------------------------------
    C14CntInit();

    ///  -----------------------------------------------------------------------
    ///  Initialization of PWM outputs
    ///  -----------------------------------------------------------------------
    C14PwmInit();

    ///  -----------------------------------------------------------------------
    ///  Initialization of analog inputs
    ///  -----------------------------------------------------------------------
    C14AdcInit();

    ///  -----------------------------------------------------------------------
    ///  Initialization of analog outputs
    ///  -----------------------------------------------------------------------
    C14AoutInit();

    return ( (DOMAIN_IO << 8) | REVISON_IO );
}

//---------------------------------------------------------------------------
//
// Function:     C14ShutDown
//
// Description:  This function disables the output driver and stops
//               peripherals.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14ShutDown (void)
{
    SPI_OE = 1;
    // stop ADC 
    C14AdcShutDown();
}

//---------------------------------------------------------------------------
//
// Function:     C14HardwareTarget
//
// Description:  This function executes an hardware reset by external watchdog.
//               The instruction trap(0) can not be done because serveral peripherals
//               have to be stoped befor instruction is executed. 
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC C14ResetTarget (void)
{
    PSW_IEN = 0;

    C14EnableWatchdog();

    // waiting for reset
	while(1);
}

//---------------------------------------------------------------------------
//
// Function:     C14GetHardwareInfo
//
// Description:  This function deliver the hardware info of the PCB
//
//
// Parameters:   pointer to info struct tC14HwInfo
//
// 
// Returns:      hardware info
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC C14GetHardwareInfo (tC14HwInfo FAR* pHwInfo_p)
{

    pHwInfo_p->m_bHwIdCpu        = C14GetCpuHWCfg();
    pHwInfo_p->m_wPcbCpuVersion  = (WORD)PCB_CPU_VERSION;
    pHwInfo_p->m_bPcbCpuRevision = (pHwInfo_p->m_bHwIdCpu & 0x0F) - 9;
    pHwInfo_p->m_bHwIdIo         = C14GetIoHWCfg ();
    pHwInfo_p->m_wPcbIoVersion   = (WORD)PCB_IO_VERSION;
    pHwInfo_p->m_bPcbIoRevision  = 8 - (pHwInfo_p->m_bHwIdIo & 0x07);

}


//---------------------------------------------------------------------------
//
// Function:     C14InitAsc0
//
// Description:  This function initializes the serial interface ASC0, if
//               is not done by other driver or monitor.
//
//
// Parameters:   Baudrate
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------
void PUBLIC C14InitAsc0 (DWORD dwBaudrate_p)
{
    // interface already initialized ?
    if (ASC0_CON != 0)
       return;

    switch (dwBaudrate_p)
    {
        //  - fractional divider as prescaler for baud rate timer is used
        default:
        case 9600L :    
            //  - required baud rate = 9,600 kbaud
            //  - real baud rate     = 9,600 kbaud
            //  - deviation          = 0,001 %
            ASC0_FDV       =  0x0074;      // load ASCx fractional divider register
            ASC0_BG        =  0x003A;     // load ASCx baud rate time reload register
            break;
        
        case 19200L :   
            //  - required baud rate = 19,200 kbaud
            //  - real baud rate     = 19,200 kbaud
            //  - deviation          = 0,001 %
            
            ASC0_FDV       =  0x00E8;      // load ASCx fractional divider register
            ASC0_BG        =  0x003A;     // load ASCx baud rate time reload register
            break;
                
        case 38400L :   
            //  - required baud rate = 38,400 kbaud
            //  - real baud rate     = 38,400 kbaud
            //  - deviation          = 0,001 %
            ASC0_FDV       =  0x01D0;     // load ASCx fractional divider register
            ASC0_BG        =  0x003A;     // load ASCx baud rate time reload register
            break;
                    
        case 57600L :   
            //  - required baud rate = 57,600 kbaud
            //  - real baud rate     = 57,592 kbaud
            //  - deviation          = -0,014 %
            ASC0_FDV       =  0x01CC;      // load ASCx fractional divider register
            ASC0_BG        =  0x0026;      // load ASCx baud rate time reload register
            break;                    
                
        case 115200L :  
            //  - required baud rate = 115,200 kbaud
            //  - real baud rate     = 115,177 kbaud
            //  - deviation          = -0,020 %
            ASC0_FDV       =  0x0191;      // load ASCx fractional divider register
            ASC0_BG        =  0x0010;      // load ASCx baud rate time reload register
            break;
    }

    P3  |= 0x0400;        // SET PORT 3.10 OUTPUT LATCH (TXD)
    ALTSEL0P3 |= 0x0C00;  // Configure port pins for serial interface 0
    DP3 |= 0x0400;        // SET PORT 3.10 DIRECTION CONTROL (TXD OUTPUT)
    DP3 &= 0xF7FF;        // RESET PORT 3.11 DIRECTION CONTROL (RXD INPUT)
    ASC0_TIC = 0x80;      // SET TRANSMIT INTERRUPT FLAG
    ASC0_RIC = 0x00;      // DELETE RECEIVE INTERRUPT FLAG
    ASC0_CON = 0x8811;    // SET SERIAL MODE
}

//---------------------------------------------------------------------------
//
// Function:     C14SetRunLED
//
// Description:  The function sets/resets the run led.
//
//
// Parameters:   State_p : state of the led
//
// 
// Returns:      none
//
//
// State:        tested (30.03.2004 A.S.)
//
//---------------------------------------------------------------------------
void PUBLIC C14SetRunLED (bit State_p)
{
    if( State_p )
    {
        // switch LED on
        CPU_RUN_LED =  LED_ON;
    }
    else
    {
        // switch LED off
        CPU_RUN_LED =  LED_OFF;
    }
}



//---------------------------------------------------------------------------
//
// Function:     C14SetErrLED
//
// Description:  The function sets/resets the error led.
//
//
// Parameters:   State_p : state of the led
//
// 
// Returns:      none
//
//
// State:        tested (30.03.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14SetErrLED (bit State_p)
{
    if( State_p )
    {
        // switch LED on
        CPU_ERR_LED =  LED_ON;
    }
    else
    {
        // switch LED off
        CPU_ERR_LED =  LED_OFF;
    }
}


//---------------------------------------------------------------------------
//
// Function:     C14EnableWatchdog
//
// Description:  The function enables the watchdog timer.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------
void PUBLIC C14EnableWatchdog (void)
{
    DP3 |= 0x0004;      // enable output driver = Watchdog enable
}

//---------------------------------------------------------------------------
//
// Function:     C14ResetWatchdog
//
// Description:  The function resets the watchdog.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------
void PUBLIC C14ResetWatchdog (void)
{
    CPU_TRG_WDT ^= 1;
}


//---------------------------------------------------------------------------
//
// Function:     C14GetSwitch
//
// Description:  The function gets the actual value of the RUN/STOP switch.
//
//
// Parameters:   none
//
// 
// Returns:      value of the switch
//                 SWITCH_MRES switch in position MRES
//                 SWITCH_STOP switch in position STOP
//                 SWITCH_RUN  switch in position RUN
//
//
// State:        tested (30.03.2004 A.S.)
//
//---------------------------------------------------------------------------
BYTE PUBLIC C14GetSwitch (void)
{
BYTE  bRetval;
WORD  wSwitch;

    // read switch state
    wSwitch  = (P5 >> 6);

    switch( wSwitch & 0x03 )
    {
        // switch in position MRES
        case 2:   bRetval = SWITCH_MRES;   
                  break;

        // switch in position RUN        
        case 3:   bRetval = SWITCH_RUN;    
                  break;

        // switch in position STOP
        case 1:
        default:
                  bRetval = SWITCH_STOP;   
                  break;
    }

    return bRetval;
}



//---------------------------------------------------------------------------
//
// Function:     C14GetHexNumber
//
// Description:  The function delivers the value of the two hex code switches.
//
//
// Parameters:   none
//
// 
// Returns:      value of the hex code switch
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------
BYTE PUBLIC C14GetHexNumber (void)
{
BYTE abRxBuff[3];
BYTE bHexCode;

    // init ressource
    // If the Register ONES is used for sending, don't increment the
    // send-pointer.
    C14SpiInitRessource (SPI_DEVICE_HWCFG_CPU, tabentries (abRxBuff), 
                         &ONES, abRxBuff, NULL);
    // start transfer
    C14SpiStartSingleCycle(SPI_DEVICE_HWCFG_CPU);

    // poll state
    while (C14SpiGetRrcState(SPI_DEVICE_HWCFG_CPU) == SPI_DEVICE_BUSY);

    // HEX-Switch: 2.Byte, change nibble
    bHexCode  = abRxBuff[1] << 4;
    bHexCode |= abRxBuff[1] >> 4;
    return (bHexCode);
}


//---------------------------------------------------------------------------
//
// Function:     C14GetDIPSwitch
//
// Description:  The function delivers the value of DIP-Switch.
//
//
// Parameters:   none
//
// 
// Returns:      value of the dipswitch
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------
BYTE PUBLIC C14GetDIPSwitch (void)
{
BYTE abRxBuff[3];

    // init ressource
    // If the Register ONES is used for sending, don't increment the
    // send-pointer.
    C14SpiInitRessource (SPI_DEVICE_HWCFG_CPU, tabentries (abRxBuff), 
                         &ONES, abRxBuff, NULL);
    // start Transfer
    C14SpiStartSingleCycle(SPI_DEVICE_HWCFG_CPU);

    // poll state
    while (C14SpiGetRrcState(SPI_DEVICE_HWCFG_CPU) == SPI_DEVICE_BUSY);

    // DIP-Switch: 1.Byte
    return (abRxBuff[0]);
}

//---------------------------------------------------------------------------
//
// Function:     C14GetBattState
//
// Description:  The function delivers the battery state.
//
//
// Parameters:   none
//
// 
// Returns:      battery state
//               TRUE  --> battery ok
//               FALSE --> battery wrong
//
//
// State:        tested (30.03.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14GetBattState (void)
{
    return(CPU_BAT_LOW?TRUE:FALSE);
}

//---------------------------------------------------------------------------
//
// Function:     C14GetHWCfg
//
// Description:  This function gets the actual value of modul configuration
//               MK0-3.
//
//
// Parameters:   none
//
// 
// Returns:      value of the configuration
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14GetIoHWCfg (void)
{
BYTE abRxBuff[1];

    // init ressource 
    // If the Register ONES is used for sending, don't increment the
    // send-pointer.
    C14SpiInitRessource (SPI_DEVICE_HWCFG_IO, tabentries (abRxBuff), 
                         &ONES, abRxBuff, NULL);
    // start transfer
    C14SpiStartSingleCycle(SPI_DEVICE_HWCFG_IO);

    // poll state
    while (C14SpiGetRrcState(SPI_DEVICE_HWCFG_IO) == SPI_DEVICE_BUSY);

    return (abRxBuff[0]);
}

//---------------------------------------------------------------------------
//
// Function:     C14GetCpuHWCfg
//
// Description:  The function gets the actual value of CPU configuration.
//
//
// Parameters:   none
//
// 
// Returns:      value of the configuration
//
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14GetCpuHWCfg (void)
{
BYTE abRxBuff[3];

    // init ressource 
    // If the Register ONES is used for sending, don't increment the
    // send-pointer.
    C14SpiInitRessource (SPI_DEVICE_HWCFG_CPU, tabentries (abRxBuff), 
                         &ONES, abRxBuff, NULL);
    // start transfer
    C14SpiStartSingleCycle(SPI_DEVICE_HWCFG_CPU);

    // poll state
    while (C14SpiGetRrcState(SPI_DEVICE_HWCFG_CPU) == SPI_DEVICE_BUSY);

    // HWCFG: 3.Byte
    return (abRxBuff[2]);
}

//---------------------------------------------------------------------------
//
// Function:     C14GetCardState
//
// Description:  The function delivers the SD-Card state.
//
//
// Parameters:   none
//
// 
// Returns:      SD-Card state
//               TRUE  --> card detected
//               FALSE --> card not detected
//
//
// State:        
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14GetCardState (void)
{
    return(CPU_SDC_DET?TRUE:FALSE);
}


//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

// ...

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 Digitale IO driver>                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static BYTE bdata bDin;

sbit   bDin_P0 = bDin^0;
sbit   bDin_P1 = bDin^1;
sbit   bDin_P2 = bDin^2;
sbit   bDin_P3 = bDin^3;
sbit   bDin_P4 = bDin^4;
sbit   bDin_P5 = bDin^5;

BYTE  abValue_l[3];

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     C14GetInGroup
//
// Description:  The function delivers the value of the digital input
//               group DI0..DI23.
//               In the MSB-Byte are placed the actual value of counter 
//               inputs too. So the counter-input also can used as simple 
//               digital input.
//
// Parameters:   pointer to process image
//               size of process image area for digital inputs
//
// 
// Returns:      number of written bytes to process image
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------
// NOTE: The following picture shows the place of each input in the DWORD.
//
// |  31  |  30  |  29  |  28  |  27  |  26  |  25  |  24  |
// +------+------+------+------+------+------+------+------+
// |  xx  |  xx  | CNT5 | CNT4 | CNT3 | CNT2 | CNT1 | CNT0 |
//
// |  23  |  22  |  21  |  20  |  19  |  18  |  17  |  16  |
// +------+------+------+------+------+------+------+------+
// | DI23 | DI22 | DI21 | DI20 | DI19 | DI18 | DI17 | DI16 | 
//
// |  15  |  14  |  13  |  12  |  11  |  10  |  09  |  08  |
// +------+------+------+------+------+------+------+------+
// | DI15 | DI14 | DI13 | DI12 | DI11 | DI10 | DI9  | DI8  | 
//
// |  07  |  06  |  05  |  04  |  03  |  02  |  01  |  00  |
// +------+------+------+------+------+------+------+------+
// | DI7  | DI6  | DI5  | DI4  | DI2  | DI2  | DI1  | DI0  | 
//
// xx -> not used 
//---------------------------------------------------------------------------

WORD PUBLIC C14GetInGroup (BYTE FAR* pbProcessImage_p, WORD wImageSize_p)
{
    ASSERT (wImageSize_p >= (tabentries(abValue_l) + 1) );

    // init ressource: send 3x 0xFF, receive 3 Bytes
    // If the Register ONES is used for sending, don't increment the
    // send-pointer.
    // For receiving use a local variable, they must not go upper the 64k-limit.
    // This can't be guaranteet at the process image.
    C14SpiInitRessource (SPI_DEVICE_DIN_SINGLE, tabentries(abValue_l), &ONES, abValue_l, NULL);

    // start transfer
    C14SpiStartSingleCycle(SPI_DEVICE_DIN_SINGLE);

    // poll status
    while (C14SpiGetRrcState(SPI_DEVICE_DIN_SINGLE) == SPI_DEVICE_BUSY);

    // read in the alternative inputs
    bDin    = 0;
    bDin_P0 = CPU_CNT0;
    bDin_P1 = CPU_CNT1;
    bDin_P2 = CPU_CNT2;
    bDin_P3 = CPU_CNT3;
    bDin_P4 = CPU_CNT4;
    bDin_P5 = CPU_CNT5;
    
    // return value of local input image
    *pbProcessImage_p = abValue_l[2];
    pbProcessImage_p++;
    *pbProcessImage_p = abValue_l[1];
    pbProcessImage_p++;
    *pbProcessImage_p = abValue_l[0];
    pbProcessImage_p++;
    *pbProcessImage_p = bDin;

    return ((tabentries(abValue_l) + 1));
}

//---------------------------------------------------------------------------
//
// Function:     C14SetOutGroup
//
// Description:  This function sets the digital outputs. The bytes 
//               are written in the following sequence:
//               - Relais0-3
//               - DO8-15
//               - DO0-7
//
// Parameters:   pointer to process image
//               size of process image area for digital outputs
// 
// Returns:      number of read bytes of process image
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

WORD PUBLIC C14SetOutGroup (BYTE FAR* pbProcessImage_p, WORD wImageSize_p)
{
DWORD dwDoutRxBuff;

    ASSERT (wImageSize_p >= 3);
    
    // Process image can exceed the 64k-limit, therefore copy it into
    // local variables.
    abValue_l[2] = *pbProcessImage_p; // DO0 - 7
    pbProcessImage_p++;
    abValue_l[1] = *pbProcessImage_p; // DO8 - 15
    pbProcessImage_p++;
    abValue_l[0] = *pbProcessImage_p; // Relais0-3, PWM outputs P0 and P1

    // PWM channels are set by this function or by PWM driver if enabled.
    if (*pbProcessImage_p & 0x10)
       CPU_PWM0 = bitPWMPortValueOne_g;
    else
       CPU_PWM0 = bitPWMPortValueZero_g;

    if (*pbProcessImage_p & 0x20)
       CPU_PWM1 = bitPWMPortValueOne_g;
    else
       CPU_PWM1 = bitPWMPortValueZero_g;
    
    // init ressource: send 3x 0xFF, receive 3 Bytes
    C14SpiInitRessource (SPI_DEVICE_DOUT, 3, abValue_l, &dwDoutRxBuff, NULL);

    // start transfer 
    C14SpiStartSingleCycle(SPI_DEVICE_DOUT);

    // poll status
    while (C14SpiGetRrcState(SPI_DEVICE_DOUT) == SPI_DEVICE_BUSY);

    return (3);
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 Analog IO driver>                              */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// The ad-converter works in continue mode with PEC-transfer at the end 
// of each channel conversion. That way the ad-conversion is independent 
// of cycle time of PLC cycle. The timing for sample and conversion can 
// be adapted (according to the source resistant) without effect to the cycle 
// time of PLC cycle. 
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//   This parameter identifies ADC fixed channel single conversion mode
#define ADC_FIXED       0

// timing AD conversion (compatibility mode)
//#define ADCTC           0x01    // conversion time control fBC = fADC/2
#define ADCTC           0x02    // conversion time control fBC = fADC/16
//#define ADSTC           0x02    // sample time control ts = tBC * 32
#define ADSTC           0x03    // sample time control ts = tBC * 64
#define ADCIN           0       // channel injection mode disabled
#define ADWR            1       // ADAT Wait enabled
#define ADM             0x02    // Auto scan single conversion selected
#define ADCH            0x03    // first channel is 3 (AIN3)

// number of ADC channel
#define ADC_CHANNEL     ADCH+1

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static WORD idata awC14ADCBuff_l[ADC_CHANNEL];

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14AdcInit
//
// Description:  The function initializes the ad-converter. 
//               The ad-converter works in continue mode with PEC-transfer
//               at the end of each channel conversion. 
//
// Parameters:   none
//               
//
// 
// Returns:      none
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14AdcInit(void)
{
    // --------------------------------------------------
    // PEC-Kanal initializing
    // - PEC channel 0 (ILVL=14, GLVL=0) is used
    // - pass 4 cycles, then request an interrupt
    // - WORD transfer, destination pointer is added each cycle
    // - source pointer points to register ADC_DAT
    // - destination pointer points to ADCBuffer
    ADC_CIC = 0x0078;
    SRCP0   = (WORD)((WORD huge*)&ADC_DAT);
    DSTP0   = (WORD)&awC14ADCBuff_l[0];
    PECSEG0 = (WORD)((DWORD)(WORD huge*)&awC14ADCBuff_l[0]>>16);
    PECC0   = 0x0200 + ADC_CHANNEL;

    //  - compatibility mode is selected
    //  - auto scan single conversion mode is selected
    //  - 10-bit resolution
    //  - ADC start bit is reset
    //  - converter basic clock tbc is fcpu / 2
    //  - sample time tsc is tbc * 64
    ADC_CON    = (ADCTC << 14)+ (ADSTC << 12)+ (ADWR << 9)+ (ADM << 4)+ ADCH;
    // start conversion
    ADC_CON_ADST = 1;
}

//---------------------------------------------------------------------------
//
// Function:     C14AdcShutDown
//
// Description:  The function finishes ad converter. 
//
// Parameters:   none
//               
//
// 
// Returns:      none
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14AdcShutDown(void)
{
    // conversion stop
    ADC_CON    = 0x0000;

    // disable PEC channel
    ADC_CIC = 0x0000;
    PECC0   = 0x0000; 
}

//---------------------------------------------------------------------------
//
// Function:     C14AdcGet
//
// Description:  This function gets the result of 4 channels.
//
// Parameters:   pointer to process image
//               size of process image area for analog inputs
//
// 
// Returns:      number of bytes written to process image
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

WORD PUBLIC C14AdcGet (BYTE FAR* pbProcessImage_p, WORD wImageSize_p)
{
    ASSERT (wImageSize_p >= sizeof(awC14ADCBuff_l));

    // write result channel AIN0 to process image
    *(WORD FAR*)pbProcessImage_p = (awC14ADCBuff_l[3] & 0x3FF) << 5;
    pbProcessImage_p += 2;

    // write result channel AIN1 to process image
    *(WORD FAR*)pbProcessImage_p = (awC14ADCBuff_l[2] & 0x3FF) << 5;
    pbProcessImage_p += 2;

    // write result channel AIN2 to process image
    *(WORD FAR*)pbProcessImage_p = (awC14ADCBuff_l[1] & 0x3FF) << 5;
    pbProcessImage_p += 2;

    // write result channel AIN3 to process image
    *(WORD FAR*)pbProcessImage_p = (awC14ADCBuff_l[0] & 0x3FF) << 5;

    return (sizeof(awC14ADCBuff_l));
}

//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14AdcIsrHandler
//
// Description:  This function is the ISR for ADC-converter. 
//               The funktion is called at the end of the PEC cycle.
//               The function starts the next conversion cycle. 
//
// Parameters:   none
//               
//
// 
// Returns:      none
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14AdcIsrHandler(void) interrupt 0x28
{
    // --------------------------------------------------
    // PEC-Kanal re-initializing
    // - PEC channel 0 (ILVL=14, GLVL=0) is used
    // - pass 4 cycles, then request an interrupt
    // - WORD transfer, destination pointer is added each cycle
    // - source pointer points to register ADC_DAT
    // - destination pointer points to ADCBuffer
    DSTP0   = (WORD)&awC14ADCBuff_l[0];
    PECC0   = 0x0200 + ADC_CHANNEL; 

    // - compatibility mode is selected
    // - auto scan single conversion mode is selected
    // - 10-bit resolution
    // - ADC start bit is reset
    // - converter basic clock tbc = fadc/x
    // - sample time tsc is tbc * 64
    ADC_CON    = (ADCTC << 14)+ (ADSTC << 12)+ (ADWR << 9)+ (ADM << 4)+ ADCH;
    // start conversion
    ADC_CON_ADST = 1; 
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 Counter driver>                                */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// The CM-C14 supports up to 4 counter inputs (3 fast counter inputs, 
// one slow counter input). The counter inputs are connected to 
// XC161-CPU counter inputs:
// fast counter: CNT0=T2, CNT1=T3, CNT2=T4
// slow counter: CNT3=T5
//
// By software the 16Bit counter inputs of CPU are extended to 32Bit counter.
//
// For each of the counter inputs it can be selected the event causing an
// increment or decrement: positiv, negativ or both transition at the 
// input line. The direction of counting can be controlled by software or
// direction input line DIN22, DIN23 (only counter CNT0 and CNT2). 
//
// direction controlling:
// C0 (T2IN): controlled by software and by hardware (DI23)
// C1 (T3IN): controlled by software
// C2 (T4IN): controlled by software and by hardware (DI22)
// C3 (T5IN): controlled by software
//
// Alternative funktion of general purpose timer:
// The timer can be used as gated timer, incremental interface inputs
// or interrupt lines.
//
// Gated Timer:
// In this mode the timer input line is used to control the gate for 
// CPU-internal timer. That way it is possible to measure the width of pulses.
//
// Incremental interface mode:
// CNT0(T2) and CNT2(T4) support together with the direction lines (CNT4, CNT5)
// the connection of an incremental counting device. The counter inputs lines
// are used as interrupt lines to detected the reference points. 
//
// operating modes:
//
// Rotation Detection: There is no interrupt request, if the direction has been 
//                     changed.
//
// Edge Detection:     Each detected edge request an interrupt.
//
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define NUMBER_OF_CNT_CHANNELS (CNT3+1)
#define TxM                     0x001       // Counter Mode

#define INT_PRIORITY_CNT0       0x0004      // ILVL = 4, GLVL = 0
#define INT_PRIORITY_CNT1       0x0005      // ILVL = 4, GLVL = 1
#define INT_PRIORITY_CNT2       0x0006      // ILVL = 4, GLVL = 2
#define INT_PRIORITY_CNT3       0x0007      // ILVL = 4, GLVL = 3

// If used only 16Bit counter, then interrupt enable flags not set. 
//#define INT_ENABLE              0x0000      // IE-Flag
#define INT_ENABLE              0x0040      // IE-Flag

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------
static int iMsbWordCnt0_l;
static int iMsbWordCnt1_l;
static int iMsbWordCnt2_l;
static int iMsbWordCnt3_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------


//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14InitCounter
//
// Description:  Diese Funktion initialisiert die Zhlervariablen. 
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (16.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14CntInit(void)
{
    C14CntSet(CNT0, 0L);
    C14CntSet(CNT1, 0L);
    C14CntSet(CNT2, 0L);
    C14CntSet(CNT3, 0L);
}

//---------------------------------------------------------------------------
//
// Function:     C14CntGet
//
// Description:  The function reads the counter values of channel 0-3. 
//
//
// Parameters:   bChannel_p : channel number
//               0 = CIN20 (timer 2),  1 = CIN21 (timer 3)
//               2 = CIN22 (timer 4),  3 = CIN23 (timer 5)
//
// 
// Returns:      value of selected counter
//               CNTRCHERR - unvalid parameter counter channel
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

long PUBLIC C14CntGet(BYTE bChannel_p)
{
    switch (bChannel_p)
    {
        //----------------------------------------------
        case CNT0:
                _atomic_(0);
                __asm
                {
                    MOV R4, GPT12E_T2      // low-Teil
                    MOV R5, iMsbWordCnt0_l // high-Teil
                }
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT1:
                _atomic_(0);
                __asm
                {
                    MOV R4, GPT12E_T3      // low-Teil
                    MOV R5, iMsbWordCnt1_l // high-Teil
                }
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT2:
                _atomic_(0);
                __asm
                {
                    MOV R4, GPT12E_T4      // low-Teil
                    MOV R5, iMsbWordCnt2_l // high-Teil
                }
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT3:
                _atomic_(0);
                __asm
                {
                    MOV R4, GPT12E_T5      // low-Teil
                    MOV R5, iMsbWordCnt3_l // high-Teil
                }
                _endatomic_();
        break;
        //----------------------------------------------
        default:
                __asm
                {
                    MOV R4, #0      // low-Teil
                    MOV R5, #0      // high-Teil
                }
        break;  
    }

    return;
}

//---------------------------------------------------------------------------
//
// Function:     C14CntSet
//
// Description:  The function pre-sets an counter channel. 
//
//
// Parameters:   bChannel_p : channel number
//               0 = CIN20 (timer 2),  1 = CIN21 (timer 3)
//               2 = CIN22 (timer 4),  3 = CIN23 (timer 5)
//
//               wChannelValue_p: start value 
//
// 
// Returns:      error code
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14CntSet(BYTE bChannel_p, long liChannelValue_p)
{
BYTE bErr = C14_SUCCESSFUL;

    switch (bChannel_p)
    {
        //----------------------------------------------
        case CNT0:
                _atomic_(0);
                GPT12E_T2      = (WORD)liChannelValue_p;
                iMsbWordCnt0_l = (int)(liChannelValue_p >> 16);
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT1:
                _atomic_(0);
                GPT12E_T3      = (WORD)liChannelValue_p;
                iMsbWordCnt1_l = (int)(liChannelValue_p >> 16);
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT2:
                _atomic_(0);
                GPT12E_T4      = (WORD)liChannelValue_p;
                iMsbWordCnt2_l = (int)(liChannelValue_p >> 16);
                _endatomic_();
        break;

        //----------------------------------------------
        case CNT3:
                _atomic_(0);
                GPT12E_T5      = (WORD)liChannelValue_p;
                iMsbWordCnt3_l = (int)(liChannelValue_p >> 16);
                _endatomic_();
        break;
        //----------------------------------------------
        default:
                bErr = C14_INVALID_CHANNEL;
        break;  
    }
    return (bErr);
}

//---------------------------------------------------------------------------
//
// Function:     C14CntSetMode
//
// Description:  The function sets the operation mode of the selected 
//               counter (direction, edge of transition).
//
//
// Parameters:   bChannel_p : channel number
//               0    = CIN20 (timer 2), 1 = CIN21 (timer 3)
//               2    = CIN22 (timer 4), 3 = CIN23 (timer 5)
//               bEdge_p:      releated edge --> TxI-Flags
//               (00 = counter disabled)
//               (01 = rising edge)
//               (10 = falling edge)
//               (11 = both edges)
//               bDir_p: direction of counting --> TxUDE, TxUD
//               (00=up,   01=down, 10=external Up/Down-control)
//               bitRun_p: Run-Bit       
//               (0=stop, 1=run)
//
// 
// Returns:      errorcode
//
//
// State:        tested (13.04.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14CntSetMode (BYTE bChannel_p, BYTE bEdge_p, BYTE bDir_p, bit bitRun_p)
{
WORD wTimerCon;

    if (!(bChannel_p < NUMBER_OF_CNT_CHANNELS))
       return (C14_INVALID_CHANNEL);

    wTimerCon = (bEdge_p & 0x03) + (TxM << 3) + ((bDir_p & 0x03) << 7);

    if (bitRun_p)
       wTimerCon |= 0x0040;

    switch (bChannel_p)
    {
        case CNT0:  GPT12E_T2CON = wTimerCon;          
                    GPT12E_T2IC  = INT_ENABLE + INT_PRIORITY_CNT0;
        break;

        case CNT1:  GPT12E_T3CON = wTimerCon & 0xFEFF; // no external up/down control
                    GPT12E_T3IC  = INT_ENABLE + INT_PRIORITY_CNT1;
        break;

        case CNT2: GPT12E_T4CON = wTimerCon;          
                   GPT12E_T4IC  = INT_ENABLE + INT_PRIORITY_CNT2;
        break;

        case CNT3: GPT12E_T5CON = wTimerCon & 0xFEFF; // no external up/down control
                   GPT12E_T5IC  = INT_ENABLE + INT_PRIORITY_CNT3;
        break;

        default: break;
    }
    return (C14_SUCCESSFUL);
}



//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14Counter0Isr
//
// Description:  The function is called if 16Bit-counter register overflows.
//               The function increments the high-part of the 32Bit counter
//               register. 
//
//
// Parameters:   none
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void C14Counter0Isr (void) interrupt 0x22
{
    // If set timer counts down
    if (GPT12E_T2CON_T2RDIR)
       iMsbWordCnt0_l += ONES; // counting down (+ (-1))
    else
       iMsbWordCnt0_l -= ONES; // counting up (- (-1))
}

//---------------------------------------------------------------------------
//
// Function:     C14Counter1Isr
//
// Description:  The function is called if 16Bit-counter register overflows.
//               The function increments the high-part of the 32Bit counter
//               register. 
//
//
// Parameters:   none
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void C14Counter1Isr (void) interrupt 0x23
{
    // If set timer counts down
    if (GPT12E_T3CON_T3RDIR)
       iMsbWordCnt1_l += ONES; // counting down (+ (-1))
    else
       iMsbWordCnt1_l -= ONES; // counting up (- (-1))
}

//---------------------------------------------------------------------------
//
// Function:     C14Counter2Isr
//
// Description:  The function is called if 16Bit-counter register overflows.
//               The function increments the high-part of the 32Bit counter
//               register. 
//
//
// Parameters:   none
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void C14Counter2Isr (void) interrupt 0x24
{
    // If set timer counts down
    if (GPT12E_T4CON_T4RDIR)
       iMsbWordCnt2_l += ONES; // counting down (+ (-1))
    else
       iMsbWordCnt2_l -= ONES; // counting up (- (-1))
}

//---------------------------------------------------------------------------
//
// Function:     C14Counter3Isr
//
// Description:  The function is called if 16Bit-counter register overflows.
//               The function increments the high-part of the 32Bit counter
//               register. 
//
//
// Parameters:   none
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void C14Counter3Isr (void) interrupt 0x25
{
    // If set timer counts down
    if (GPT12E_T5CON_T5UD)
       iMsbWordCnt3_l += ONES; // counting down (+ (-1))
    else
       iMsbWordCnt3_l -= ONES; // counting up (- (-1))
}


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 AOUT driver>                                   */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// The analog voltage are generated by an PWM signal with activ low pass 
// filter (1. order). 
// The frequency of PWM signal is about 39kHz. The 3dB-frequency of low pass
// filter is 53 Hz. The pulse width of PWM signal is controlled with 10Bit. 
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     C14AoutInit
//
// Description:  The function initializes the both PWM channels (P2, 3)
//               for using as analog outputs. The start value of analog 
//               output voltage is 0V (according to 0% puls width of PWM signal).
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14AoutInit (void)
{
    // select the periode time of base timer for resolution of 10Bit
    // to 25s, the periode time of PWM signal is 1024 * 25s
    C14PwmPtoTimer (PWM2, 0x000);
    // set periode of PWM signal, set start value of PWM pulse to 0%
    C14PwmParam   (PWM2, 1024, 0);
    C14PwmParam   (PWM3, 1024, 0);
    // start PWM channels
    C14PwmEnable  (PWM2, TRUE);
    C14PwmEnable  (PWM3, TRUE);
}

//---------------------------------------------------------------------------
//
// Function:     C14AoutSet
//
// Description:  The function sets a new value to analog output channel. 
//
// Parameters:   pointer to process image
//               size of process image area for digital outputs
// 
// Returns:      number of read bytes of process image
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

WORD PUBLIC C14AoutSet (BYTE FAR* pbProcessImage_p, WORD wImageSize_p)
{
WORD wPulse;

    ASSERT (wImageSize_p >= 4);

    wPulse = *(WORD FAR*)pbProcessImage_p;
    wPulse <<= 1;
    C14PwmParam (PWM2, 1024, wPulse);
    pbProcessImage_p+=2;

    wPulse = *(WORD FAR*)pbProcessImage_p;
    wPulse <<= 1;
    C14PwmParam (PWM3, 1024, wPulse);

    return (4);
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 SPI driver>                                    */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// 
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define MAX_NUMBER_OF_PEC_SERVICES  0xF0

// constant for the SPI baudrate reload register
// for a fssc=40MHz
#define SPI_1MBAUD          0x13
#define SPI_2MBAUD          0x09
#define SPI_4MBAUD          0x04
#define SPI_5MBAUD          0x03

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

typedef struct
{
    // general parameters for the selected SPI device
    BYTE        m_bDeviceAddr;    // Address of SPI device 
    WORD        m_wSpiBaudrate;   // spi device baudrate
    WORD        m_wSpiCntrl;      // value of the control register
    tfpC14SpiCb m_fpC14SpiCb;     // Callback-Funktion for signal events to application

    // parameters of application for a transfer sequence via PEC-channel
    BYTE        m_bState;         // state of channel
    WORD        m_wPECC2;         // value of PEC control register 2
    WORD        m_wSRCP2;         // PEC source pointer offset of channel 2
    WORD        m_wDSTP2;         // PEC destination pointer offset of channel 2
    WORD        m_wPECSEG2;       // PEC source and dest pointer segment channel 2
    WORD        m_wPECC1;         // value of PEC control register 1
    WORD        m_wSRCP1;         // PEC source pointer offset of channel 1
    WORD        m_wDSTP1;         // PEC destination pointer offset of channel 1
    WORD        m_wPECSEG1;       // PEC source and dest pointer segment channel 1

} tSpiRessource;

//---------------------------------------------------------------------------
// local vars
//---------------------------------------------------------------------------

static const tSpiRessource RessourceTable[] =
{
    // continue reading of digital inputs, writing LEDs
    {SPI_CS_DIN_LED,  SPI_2MBAUD, 0x4237, NULL, 0, 0x0000, 0, 0, 0, 0x0000, 0, 0, 0}, 
    // single cycle reading of digital inputs, writing LEDs, PEC: source pointer not incr.
    {SPI_CS_DIN_LED,  SPI_2MBAUD, 0x4237, NULL, 0, 0x0100, 0, 0, 0, 0x0300, 0, 0, 0}, 
    // writing digitale outputs, LEDs
    {SPI_CS_DOUT_LED, SPI_2MBAUD, 0x4237, NULL, 0, 0x0500, 0, 0, 0, 0x0300, 0, 0, 0},
    // reading HW configuration of IO board, PEC: source pointer not incr.
    {SPI_CS_HWCFG_IO, SPI_2MBAUD, 0x4237, NULL, 0, 0x0100, 0, 0, 0, 0x0300, 0, 0, 0},
    // reading DIP-Switch, HEX-Switch, configuration of CPU board, PEC: source pointer not incr.
    {SPI_CS_CODE,     SPI_2MBAUD, 0x4237, NULL, 0, 0x0100, 0, 0, 0, 0x0300, 0, 0, 0},
    // reading and writing SPI-EEPROM
    {SPI_CS_EEPR1,    SPI_2MBAUD, 0x4237, NULL, 0, 0x0500, 0, 0, 0, 0x0300, 0, 0, 0},
    // reading and writing SD-Card-Connector
    {SPI_CS_SD,       SPI_2MBAUD, 0x4237, NULL, 0, 0x0500, 0, 0, 0, 0x0300, 0, 0, 0},
    // reading and writing EEPROM2 CPU board
    {SPI_CS_EEPR2,    SPI_2MBAUD, 0x4237, NULL, 0, 0x0500, 0, 0, 0, 0x0300, 0, 0, 0}
};

static tSpiRessource aRessourceTable_l[tabentries(RessourceTable)];

static BYTE bDevice_l;

static bit  bitRestartContinueCycle_l;

static tfpC14SpiCbCriticalSection fpC14SpiCriticalSection_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------
static void C14SpiSelectDevice            (BYTE bDeviceAddress_p);
static void C14SpiStartContinueCycle      (void);
static void C14SpiStopContinueCycle       (void);
void        C14SpiTransferSequence        (BYTE bRrcIdx_p);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//


//---------------------------------------------------------------------------
//
// Function:     C14SpiInit
//
// Description:  This function initializes the SSC0 configuration for IO 
//               access.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14SpiInit (void)
{
WORD  i;

    // Ressourcentabelle init. 
    for (i=0; i < tabentries(aRessourceTable_l); i++)
        aRessourceTable_l[i] = RessourceTable[i];

    //-----------------------------------------------------------------------
    // set address decoder
    SPI_SELECT(SPI_CS_INACTIV);

    // SSC0 init.
    SSC0_CON_EN  = 0;           // disablen
    SSC0_CON     = 0x4237;      // receive error enable
    SSC0_BR      = SPI_2MBAUD;
    SSC0_CON_EN  = 1;           // enablen

    //-----------------------------------------------------------------------
    // set interrupt control
    SSC0_TIC     = 0x003A; // PEC channel 2, ILVL=14, GLVL=2, IE=0
    SSC0_RIC     = 0x003C; // PEC channel 4, ILVL=15, GLVL=0, IE=0
//    SSC0_EIC     = 0x007D; // ILVL=15, GLVL=1, IE=1
    
    //-----------------------------------------------------------------------
    // starts continue transfer: DIN -> LEDs
    C14SpiStartContinueCycle();        
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiRegisterCbCriticalSection
//
// Description:  This function sets the callback function to define a
//               critical section using SPI ressources within different tasks. 
//               Setting an NULL pointer will disable the calling of callback 
//               function.
//
//
// Parameters:   - callback-function
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC C14SpiRegisterCbCriticalSection(tfpC14SpiCbCriticalSection fpC14SpiCriticalSection_p)
{
    _atomic_(0);
    fpC14SpiCriticalSection_l = fpC14SpiCriticalSection_p;
    _endatomic_();
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiInitRessource
//
// Description:  This function writes parameter to ressource table to prepare 
//               a transfer sequence. The parameter are converted to the 
//               format of PEC register.
//
//
// Parameters:   - Index of ressource
//               - number of bytes
//               - pointer to source data
//               - pointer to destination data
//               - callback-function
//
// 
// Returns:      Errorcode
//               0x00 - successful
//              >0x00 - ressource not available
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14SpiInitRessource (BYTE bRrcIdx_p, BYTE bEntries_p, 
                                 void huge* pbTxBuff_p, void huge* pbRxBuff_p,
                                 tfpC14SpiCb fpC14SpiCb_p)
{
    if (bRrcIdx_p >= tabentries(aRessourceTable_l))
        return (SPI_DEVICE_UNAVAILABLE);

    if ( !bEntries_p || (bEntries_p > MAX_NUMBER_OF_PEC_SERVICES) )
        return (SPI_DEVICE_PARAM_ERROR);

    aRessourceTable_l[bRrcIdx_p].m_fpC14SpiCb = fpC14SpiCb_p;

    // The pointer to data source is converted to PEC register format.
    // That the way conversion, running of ISR, must not to be done. 
    // -------------------------------------------------------------
    // PECSEG register: source-seg (Bit15-8)| dest-seg (Bit7-0)
    // sending of data: transmit buffer is source of PEC transfer, register 
    // SSC0_TB is destination of PEC transfer
    aRessourceTable_l[bRrcIdx_p].m_wPECC2   &= 0xFF00; // clear last value
    aRessourceTable_l[bRrcIdx_p].m_wPECC2   += (WORD)bEntries_p;
    aRessourceTable_l[bRrcIdx_p].m_wSRCP2    = (WORD)(pbTxBuff_p);
    aRessourceTable_l[bRrcIdx_p].m_wDSTP2    = (WORD)((WORD huge*)&SSC0_TB);
    aRessourceTable_l[bRrcIdx_p].m_wPECSEG2  = ((WORD)((DWORD)pbTxBuff_p >> 8)) & 0xFF00;
    aRessourceTable_l[bRrcIdx_p].m_wPECSEG2 += 0x0000; // segment address of SFRs

    // -------------------------------------------------------------
    // receiving of data: receive buffer is destination of PEC transfer, 
    // register SSC0_RB is source of PEC transfer
    aRessourceTable_l[bRrcIdx_p].m_wPECC1   &= 0xFF00; // clear last value
    aRessourceTable_l[bRrcIdx_p].m_wPECC1   += (WORD)bEntries_p;
    aRessourceTable_l[bRrcIdx_p].m_wSRCP1    = (WORD)((WORD huge*)&SSC0_RB);
    aRessourceTable_l[bRrcIdx_p].m_wDSTP1    = (WORD)pbRxBuff_p;
    aRessourceTable_l[bRrcIdx_p].m_wPECSEG1  = ((WORD)((DWORD)pbRxBuff_p >> 16) & 0x00FF);
    aRessourceTable_l[bRrcIdx_p].m_wPECSEG1 += 0x0000; // segment address of SFRs

    return (SPI_DEVICE_SUCCESSFUL);
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiGetRrcState
//
// Description:  This function returns the transfer state of a SPI device.
//
//
// Parameters:   Index of Ressource
//
// 
// Returns:      state
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

BYTE PUBLIC C14SpiGetRrcState(BYTE bRrcIdx_p)
{
BYTE bState;
    ASSERT (bRrcIdx_p < tabentries(aRessourceTable_l));

    bState = aRessourceTable_l[bRrcIdx_p].m_bState;

    if (bState != SPI_DEVICE_BUSY )
    {
       // Access to SPI ressource finished.
       // Ends an critical section.
       if (fpC14SpiCriticalSection_l)
          fpC14SpiCriticalSection_l(bRrcIdx_p, SPI_END_CRITICAL_SECTION);
    }
    // return the state of channel: released, busy
    return (bState);
}


//---------------------------------------------------------------------------
//
// Function:     C14SpiStartSingleCycle
//
// Description:  This function starts a transfer sequence. The PEC
//               service for the SPI interface is init. The function is
//               called from interrupt.
//
//
// Parameters:   Index of Ressource
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14SpiStartSingleCycle(BYTE bRrcIdx_p)
{
    ASSERT (bRrcIdx_p < tabentries(aRessourceTable_l));

    C14SpiStopContinueCycle();
    // starts an critical section to prevent access to
    // SPI ressource by another task
    if (fpC14SpiCriticalSection_l)
       fpC14SpiCriticalSection_l(bRrcIdx_p, SPI_START_CRITICAL_SECTION);

    // The SPI interface is not allocated by another ressource. 
    // The next sequences can started. 
    C14SpiSelectDevice(aRessourceTable_l[bRrcIdx_p].m_bDeviceAddr);
    // set the state to signal that interface works
    aRessourceTable_l[bRrcIdx_p].m_bState = SPI_DEVICE_BUSY;
    // store the ressource 
    bDevice_l    = bRrcIdx_p;
    // start Callback-function
    if (aRessourceTable_l[bRrcIdx_p].m_fpC14SpiCb)
        aRessourceTable_l[bRrcIdx_p].m_fpC14SpiCb(SPI_DEVICE_TRANSFER_STARTING);

    // send channel:
    SRCP2   = aRessourceTable_l[bRrcIdx_p].m_wSRCP2;
    DSTP2   = aRessourceTable_l[bRrcIdx_p].m_wDSTP2;
    PECSEG2 = aRessourceTable_l[bRrcIdx_p].m_wPECSEG2;
    PECC2   = aRessourceTable_l[bRrcIdx_p].m_wPECC2;
    SSC0_TIC_IR  = 1;

    // receive channel:
    SRCP4   = aRessourceTable_l[bRrcIdx_p].m_wSRCP1;
    DSTP4   = aRessourceTable_l[bRrcIdx_p].m_wDSTP1;
    PECSEG4 = aRessourceTable_l[bRrcIdx_p].m_wPECSEG1;
    PECC4   = aRessourceTable_l[bRrcIdx_p].m_wPECC1;
    SSC0_RIC_IR  = 0;
    _nop_();
    SSC0_RIC_IE  = 1;

    #ifdef SPI_EVENT_MODE
    // ---------------------------------------------------------------------
    // Durch Setzen des Interrupt Request Flags fr den Sendekanal 
    // den ersten Transfer auslsen. 
    // Die Reihenfolge ist hier entscheidend! Zuerst das IR-Flag setzen,
    // das fhrt zu keiner nderung des Status falls es schon gesetzt war.
    // Dann den Interrupt freigeben. Dadurch wird der PEC-Service ausgefhrt.
    // Wird die Reihenfolge getauscht, so werden zwei PEC-Service ausgefhrt.
    SSC0_TIC_IE  = 1;

    #else
    // ---------------------------------------------------------------------
    // SPI device is used in polling mode.
    C14SpiTransferSequence(bRrcIdx_p);

    #endif
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiStopSingleCycle
//
// Description:  This function finishs a transfer sequence. 
//
//
// Parameters:   Index of Ressource
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14SpiStopSingleCycle(BYTE bRrcIdx_p)
{
    ASSERT (bRrcIdx_p < tabentries(aRessourceTable_l));

    // Callback-Funktion ausfhren falls vorhanden
    if (aRessourceTable_l[bRrcIdx_p].m_fpC14SpiCb)
        aRessourceTable_l[bRrcIdx_p].m_fpC14SpiCb(SPI_DEVICE_TRANSFER_FINISHED);

    // Bausteinauswahl inaktiv
    SPI_SELECT(SPI_CS_INACTIV);

    // set status for this source to signal application that sequence is finished
    aRessourceTable_l[bRrcIdx_p].m_bState = SPI_DEVICE_RELEASED;
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiStartContinueCycle
//
// Description:  This function starts a continue transfer sequence. The PEC
//               service for the SPI interface is init. The function is
//               called from interrupt.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14SpiStartContinueCycle(void)
{
    #ifdef SPI_EVENT_MODE

    // Die SPI-Schnittstelle ist jetzt frei, die nchste Sequence
    // kann gestartet werden. Der PEC-Service ist zu init.
    // Baustein auswhlen

#pragma WARNING DISABLE = 177   // warning <unreachable code> = OFF
    // Eingnge in Shift Register lesen
    SPI_PL = 0;
    SPI_PL = 1;
    // CS aktiv
    SPI_SELECT(SPI_CS_DIN_LED);
//#pragma WARNING ENABLE = 177    // warning <unreachable code> = ON

    // set status for this source to signal application that sequence is finished
    aRessourceTable_l[SPI_DEVICE_DIN_CONTINUE].m_bState = SPI_DEVICE_BUSY;

    bDevice_l = SPI_DEVICE_DIN_CONTINUE;

    bitRestartContinueCycle_l = TRUE;

    // Sendekanal parametrieren
    SRCP2   = (WORD)((WORD huge*)&ONES);
    DSTP2   = (WORD)((WORD huge*)&SSC0_TB);
    PECSEG2 = 0x0000;   // Register liegen beide im ersten Segment
    PECC2   = 0x0103;   // 3 Transfers, PEC-Service: BYTE-Transfer

    // Empfangskanal wird verwendet, um das CS kurz inaktiv zu setzen. 
    // Dadurch werden die Daten in das Ausgangs-Latch bernommen.
    // Empfangskanal parametrieren
    SRCP4   = (WORD)((WORD huge*)&SSC0_RB);
    DSTP4   = (WORD)((WORD huge*)&ZEROS);
    PECSEG4 = 0x0000;
    PECC4   = 0x0103;
    SSC0_RIC_IR  = 0;
    _nop_();
    SSC0_RIC_IE  = 1;

    // Die Reihenfolge ist hier entscheidend! Zuerst das IR-Flag setzen,
    // das fhrt zu keiner nderung des Status falls es schon gesetzt war.
    // Dann den Interrupt freigeben. Dadurch wird der PEC-Service ausgefhrt.
    // Wird die Reihenfolge getauscht, so werden zwei PEC-Service ausgefhrt.
    SSC0_TIC_IR  = 1;
    _nop_(); // Setzen des Flags abwarten
    SSC0_TIC_IE  = 1;

    #endif

    return;
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiStopContinueCycle
//
// Description:  This function finishs a continue transfer sequence. 
//
//
// Parameters:   Index of Ressource
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14SpiStopContinueCycle(void)
{
    // disable restart of continue transfer
    bitRestartContinueCycle_l = FALSE;

    // waiting until tarnsfer is finished
    while (aRessourceTable_l[SPI_DEVICE_DIN_CONTINUE].m_bState == SPI_DEVICE_BUSY);
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiTransferSequence
//
// Description:  This function transfers a sequence of bytes. 
//
//
// Parameters:   Index of Ressource
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

void PUBLIC C14SpiTransferSequence(BYTE bRrcIdx_p)
{
WORD i;
DWORD pbSpi_MTSR_Src;
WORD  wMTSR_Con, wMRST_Con;
DWORD pbSpi_MRST_Dst;

    // disable send and receive interrupts for
    // polling mode
    SSC0_TIC_IE = 0;
    SSC0_RIC_IE = 0;

    // get the number to be transferred bytes
    i = aRessourceTable_l[bRrcIdx_p].m_wPECC1 & 0x00FF;
    //-----------------------------------------------------------------------
    // calculate source pointer
    pbSpi_MTSR_Src  = (DWORD)aRessourceTable_l[bRrcIdx_p].m_wSRCP2;
    pbSpi_MTSR_Src += ((DWORD)(aRessourceTable_l[bRrcIdx_p].m_wPECSEG2 & 0xFF00) << 8);
    wMTSR_Con       = aRessourceTable_l[bRrcIdx_p].m_wPECC2 & 0x0600;

    //-----------------------------------------------------------------------
    // calculate destination pointer
    pbSpi_MRST_Dst  = (DWORD)aRessourceTable_l[bRrcIdx_p].m_wDSTP1;
    pbSpi_MRST_Dst += ((DWORD)(aRessourceTable_l[bRrcIdx_p].m_wPECSEG1 & 0x00FF) << 16);
    wMRST_Con       = aRessourceTable_l[bRrcIdx_p].m_wPECC1 & 0x0600;

    //-----------------------------------------------------------------------
    // transfer data
    while (i)
    {
        SSC0_TB = *(BYTE huge*)pbSpi_MTSR_Src;
        while (!SSC0_TIC_IR);
        SSC0_TIC_IR = 0;

        while (!SSC0_RIC_IR);
        *(BYTE huge*)pbSpi_MRST_Dst = (BYTE)SSC0_RB;
        SSC0_RIC_IR = 0;

        // source pointer incr. ?
        if (wMTSR_Con & 0x0400)
           pbSpi_MTSR_Src++;

        // destination pointer incr. ?
        if (wMRST_Con & 0x0200)
           pbSpi_MRST_Dst++;

        i--;
    }

    SPI_SELECT(SPI_CS_INACTIV);

    // ------------------------------------------------------
    // stop transfer
    aRessourceTable_l[bRrcIdx_p].m_bState = SPI_DEVICE_RELEASED;
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiSelectDevice
//
// Description:  This function is called to select an defined SPI
//               device.
//
//
// Parameters:   Device Address
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14SpiSelectDevice(BYTE bDeviceAddress_p)
{
    // update shift register 
    SPI_PL = 0;
    SPI_PL = 1;
    __asm 
    {
        MOV  R4, R8
        BMOV P2.8,  R4.0
        BMOV P2.9,  R4.1
        BMOV P2.10, R4.2
        BMOV P2.11, R4.3
    }
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiTxHandler
//
// Description:  This function is called by interrupt if all bytes 
//               transmitted.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14SpiTxHandler(void) interrupt 0x2D
{
    // disable send interrupt
    SSC0_TIC_IE = 0;
}

//---------------------------------------------------------------------------
//
// Function:     C14SpiRxHandler
//
// Description:  This function is called by interrupt if all bytes 
//               received.
//
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        tested (15.04.2004 A.S.)
//
//---------------------------------------------------------------------------

static void C14SpiRxHandler(void) interrupt 0x2E
{
    SPI_SELECT(SPI_CS_INACTIV);

    // restart continue cycle?
    if (bitRestartContinueCycle_l)
    {
       SSC0_TIC_IE  = 0;
       SSC0_TIC_IR  = 0;
       // update shift register
       SPI_PL = 0;
       SPI_PL = 1;
       // CS activ
       SPI_SELECT(SPI_CS_DIN_LED);
       // Src und Dst-Pointer are not changed.
       PECC2   = 0x0103;   // 3 Transfers, PEC-Service: BYTE-Transfer
       PECC4   = 0x0103;

       SSC0_TIC_IR  = 1;
       SSC0_TIC_IE  = 1;

       return;
    }
  
    // ------------------------------------------------------
    // stop single cycle Transfer or cont. transfer 
    // set status for this source to signal application that sequence is finished
    aRessourceTable_l[bDevice_l].m_bState = SPI_DEVICE_RELEASED;

    // Re-start single sycle transfer. 
    if (bDevice_l != SPI_DEVICE_DIN_CONTINUE)
      C14SpiStartContinueCycle();
}


//---------------------------------------------------------------------------
//
// Function:     C14SpiErrorHandler
//
// Description:  This function is called by interrupt if one bytes 
//               lost. Function repeats the transmission.
//               Wird die Programmausfhrung durch einen Breakpoint 
//               gestoppt, so werden alle ISR gesperrt. Das Senden
//               von BYTEs luft jedoch weiter, da der Sendepuffer 
//               gepuffert ist. Da das Empfangen and das Senden gekoppelt
//               ist, werden natrlich BYTEs empfangen, jedoch kein Interrupt
//               ausgelst, um diese Bytes abzuholen. Dadurch geht ein
//               BYTE verloren und der Transfer-Zyklus kann nicht beendet 
//               werden.
//               Ein bertragungszyklus wird daher wiederholt.
//
// Parameters:   none
//
// 
// Returns:      none
//
//
// State:        
//
//---------------------------------------------------------------------------

static void C14SpiErrorHandler(void) interrupt 0x2F
{
BYTE bSelectedDevice = bDevice_l;

    SSC0_CON_RE  = 0;   // Fehlerbit lschen
    SSC0_RIC_IE  = 0;
    SSC0_TIC_IE  = 0;
    
    C14SpiStopSingleCycle(bSelectedDevice);

    switch (bSelectedDevice)
    {
        case SPI_DEVICE_DIN_CONTINUE:
            SRCP2   = (WORD)((WORD huge*)&ONES);
            DSTP2   = (WORD)((WORD huge*)&SSC0_TB);
            PECSEG2 = 0x0000;
            PECC2   = 0x0103;
        
            SRCP4   = (WORD)((WORD huge*)&SSC0_RB);
            DSTP4   = (WORD)((WORD huge*)&ZEROS);
            PECSEG4 = 0x0000;
            PECC4   = 0x0103;
        break;

        case SPI_DEVICE_DIN_SINGLE:
        case SPI_DEVICE_DOUT:
        case SPI_DEVICE_HWCFG_IO:
        case SPI_DEVICE_HWCFG_CPU:
        case SPI_DEVICE_EEPROM:
        case SPI_DEVICE_SD_CARD:

            C14SpiSelectDevice(aRessourceTable_l[bSelectedDevice].m_bDeviceAddr);
            aRessourceTable_l[bSelectedDevice].m_bState = SPI_DEVICE_BUSY;
            bDevice_l    = bSelectedDevice;
        
            SRCP2   = aRessourceTable_l[bSelectedDevice].m_wSRCP2;
            DSTP2   = aRessourceTable_l[bSelectedDevice].m_wDSTP2;
            PECSEG2 = aRessourceTable_l[bSelectedDevice].m_wPECSEG2;
            PECC2   = aRessourceTable_l[bSelectedDevice].m_wPECC2;
            SSC0_TIC_IR  = 1;
        
            SRCP4   = aRessourceTable_l[bSelectedDevice].m_wSRCP1;
            DSTP4   = aRessourceTable_l[bSelectedDevice].m_wDSTP1;
            PECSEG4 = aRessourceTable_l[bSelectedDevice].m_wPECSEG1;
            PECC4   = aRessourceTable_l[bSelectedDevice].m_wPECC1;
            SSC0_RIC_IR  = 0;
            _nop_();
            SSC0_RIC_IE  = 1;
        
            // ---------------------------------------------------------------------
            SSC0_TIC_IE  = 1;

        break;
    }
}



// EOF

