/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Produktlinie C14

  Description:  include file for io driver api function 

  -------------------------------------------------------------------------

                $RCSfile: C14Drv.h,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1 $  $Date: 2004/10/06 11:25:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/10/28 -ct:   start of the implementation, version 1.00

****************************************************************************/

#ifndef _C14DRV_H_
#define _C14DRV_H_


//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//  version number
#define DOMAIN_IO      1
#define REVISON_IO     0

// const for the switch of the led's
// ----------------------------------
#define ON              0x01            // LED on
#define OFF             0x00            // LED off

// const for the pooling of the RUN/STOP switch
// --------------------------------------------
#define SWITCH_MRES     0x01            // switch in position MRES
#define SWITCH_STOP     0x02            // switch in position STOP
#define SWITCH_RUN      0x03            // switch in position RUN

// value of reference voltage source for analouge inputs
//------------------------------------------------------
#define VREF            4.141

// value for 1 LSB
// ---------------
#define ONE_LSB         2.4412 * VREF / 32768.0


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 Dedicated functions>                           */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// typedef
//---------------------------------------------------------------------------
typedef struct
{
    WORD    m_wPcbCpuVersion;
    BYTE    m_bPcbCpuRevision;
    BYTE    m_bHwIdCpu;
    WORD    m_wPcbIoVersion;
    BYTE    m_bPcbIoRevision;
    BYTE    m_bHwIdIo;

} tC14HwInfo;



//---------------------------------------------------------------------------
// function prototypes
//---------------------------------------------------------------------------

WORD  C14Initialize         (void);
void  C14GetHardwareInfo    (tC14HwInfo FAR* pHwInfo_p);
void  C14InitAsc0           (DWORD dwBaudrate_p);
void  C14SetRunLED          (bit State_p);
void  C14SetErrLED          (bit State_p);
void  C14ResetWatchdog      (void);
BYTE  C14GetSwitch          (void);
BYTE  C14GetHexNumber       (void);
BYTE  C14GetDIPSwitch       (void);
BYTE  C14GetIoHWCfg         (void);
BYTE  C14GetCpuHWCfg        (void);
BYTE  C14GetBattState       (void);

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 Process peripherals>                           */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// error codes
#define C14_SUCCESSFUL          0x00
#define C14_BUFFER_OVERRUN      0xFC
#define C14_INVALID_PARAMETER   0xFD
#define C14_INVALID_TB          0xFE
#define C14_INVALID_CHANNEL     0xFF


// const for counter 
#define CNTRCHERR               0x0000

// number of counter channels
#define CNT0                    0x00
#define CNT1                    0x01
#define CNT2                    0x02
#define CNT3                    0x03

// parameter of counter 
#define DISABLE_EDGE            0x00    // counter stop
#define RISING_EDGE             0x01    // counting edge = rising edge
#define FALLING_EDGE            0x02    // counting edge = falling Flanke
#define ANY_EDGE                0x03    // counting edge = rising and falling edge
#define COUNTING_UP             0x00    // counting direction
#define COUNTING_DOWN           0x01    // counting direction
#define EXT_UP_DOWN_CONTROL     0x02    // counting direction: controled by external line
                                        // external line = 0: upwards
                                        // external line = 1: downwards
#define COUNTER_RUN             0x01
#define COUNTER_STOP            0x00

//---------------------------------------------------------------------------
// const for PWM
#define PWM0                    0x00
#define PWM1                    0x01
#define PWM2                    0x02
#define PWM3                    0x03

#define PTO0                    0x00
#define PTO1                    0x01

#define PPW_TB_MICRO_SEC        0x05    // base clock = 800ns
#define PPW_TB_MILLI_SEC        0x0F    // base clock = 1ms

#define CHN_OVERRUN             0x01
#define CHN_ABORT               0x02
#define CHN_DELTA_ERROR         0x04
#define CHN_BUSY                0x80
//---------------------------------------------------------------------------
// const for AOUT
#define AOUT0                   0x00
#define AOUT1                   0x01

//---------------------------------------------------------------------------
// typedef
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// function prototypes
//---------------------------------------------------------------------------

WORD  C14GetInGroup         (BYTE FAR* pbProcessImage_p, WORD wImageSize_p);
WORD  C14SetOutGroup        (BYTE FAR* pbProcessImage_p, WORD wImageSize_p);

void  C14AdcInit            (void);
void  C14AdcShutDown        (void);
WORD  C14AdcGet             (BYTE FAR* pbProcessImage_p, WORD wImageSize_p);

void  C14CntInit            (void);
BYTE  C14CntSetMode         (BYTE bChannel_p, BYTE bEdge_p, BYTE bDir_p, bit bitRun_p);
DWORD C14CntGet             (BYTE bChannel_p);
BYTE  C14CntSet             (BYTE bChannel_p, DWORD dwChannelValue_p);

void  C14PwmInit            (void);
BYTE  C14PwmPtoTimer        (BYTE bChannel_p, BYTE bScale_p);
BYTE  C14PwmParam           (BYTE bChannel_p, WORD  wPeriode_p, WORD  wPulse_p);
BYTE  C14PwmEnable          (BYTE bChannel_p, bit bitRun_p);
BYTE  C14PwmState           (BYTE bChannel_p, BYTE *pbState_p);

BYTE  C14PtoParam           (BYTE bChannel_p, WORD wPeriode_p, int iDelta_p, DWORD dwImpulse_p);
BYTE  C14PtoEnable          (BYTE bChannel_p, bit bitRun_p);

void  C14AoutInit           (void);
WORD  C14AoutSet            (BYTE FAR* pbProcessImage_p, WORD wSize_p);

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <C14 SPI driver>                                    */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

// SPI device index and state
// ---------------------------
#define SPI_DEVICE_DIN_CONTINUE      0x00
#define SPI_DEVICE_DIN_SINGLE        0x01
#define SPI_DEVICE_DOUT              0x02
#define SPI_DEVICE_HWCFG_IO          0x03
#define SPI_DEVICE_HWCFG_CPU         0x04
#define SPI_DEVICE_EEPROM            0x05
#define SPI_DEVICE_SD_CARD           0x06
#define SPI_DEVICE_EEPROM2           0x07

#define SPI_DEVICE_SUCCESSFUL        0x00
#define SPI_DEVICE_RELEASED          0x00
#define SPI_DEVICE_BUSY              0x01
#define SPI_DEVICE_PARAM_ERROR       0xFE
#define SPI_DEVICE_UNAVAILABLE       0xFF

// Events for calling user callback function
#define SPI_DEVICE_TRANSFER_STARTING 0x00   // transfer is started (CS is activ)
#define SPI_DEVICE_TRANSFER_FINISHED 0x01   // transfer is finished (CS is still activ)

//---------------------------------------------------------------------------
// typedef
//---------------------------------------------------------------------------

typedef void (PUBLIC ROM* tfpC14SpiCb)    (BYTE bEvent_p);

//---------------------------------------------------------------------------
// function prototypes
//---------------------------------------------------------------------------
void PUBLIC C14SpiInit (void);

BYTE PUBLIC C14SpiInitRessource (BYTE bRrcIdx_p, BYTE bEntries_p, 
                                 void huge* pbTxBuff_p, void huge* pbRxBuff_p,
                                 tfpC14SpiCb fpC14SpiCb_p);

void PUBLIC C14SpiStartSingleCycle(BYTE bRrcIdx_p);

BYTE PUBLIC C14SpiGetRrcState   (BYTE bRrcIdx_p);

#endif  // #ifndef _C14DRV_H_


