/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      PLCmodule C14

  Description:  example for using the power fail driver

  -------------------------------------------------------------------------

                $RCSfile: PwrfDemo.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.5 $  $Date: 2005/01/26 12:20:34 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/02/12 -as:   Start der Implementierung

****************************************************************************/

#include <xc161.h>

#include "generic/global.h"
#include "../include/PwrfDrv.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

static BYTE bPowerFailMonitor_l;
static BYTE bPowerFailAppState_l;
static BYTE bRunState_l;

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

void C14EnableWatchdog(void);
void C14ResetWatchdog (void);

//-------------------------------------------------------------------------

static void PowerFailHandler(bit bitCriticalSection_p)
{
    bRunState_l = 0;

    // The caller function disabled the power fail
    // interrupt to prevent recalling PowerFailHandler.

    // Power fail during a critical section is 
    // signaled by bitCriticalSection_p = 1.
    if (bitCriticalSection_p)
    {
       // The application runs through a critical section.
       // The handler should be finished as fast as possible
       // to leave the critical section befor the voltage is done.
       // To arrive at more time for critical section the handler
       // can disable some peripherials to save current consumption.
       // After leaving the critical section the handler is recalled
       // with bitCriticalSection_p = 0.
       return;
    }
    else
    {
       // Power fail is signaled. Application should wait for RESET.
       // For diagnostics the handler can mark the event and 
       // returns to application. The power fail interrupt 
       // will be disabled until the application re-enabled
       // the interrupt again (the event was processed).
       if (bPowerFailMonitor_l)
       {
           // mark the event for processing by application 
           // and return to application
           bPowerFailAppState_l = 1;
           return;
       }

       // While the power-fail-state, wait in idle mode 
       // Polls every 1ms the actual state
       while (PowerFailGetPowerFailState())
       {
         C14ResetWatchdog();
       }
  
       // Power Fail inactiv ---> RESET 
       C14EnableWatchdog();
       // waiting for reset done
       while(1);
    }
}
//-------------------------------------------------------------------------
void  main (void)
{
    bPowerFailMonitor_l  = 1;
    bPowerFailAppState_l = 0;
    bRunState_l          = 1;

    PSW_IEN = 1;

    PowerFailInit();

    PowerFailInstallHandler(PowerFailHandler);

    //----------------------------------------------------------------------
    // Power Fail handler acts as power fail monitor
    PowerFailStartCriticalSection();

    while (bRunState_l);

    PowerFailEndCriticalSection();

    while(PowerFailGetPowerFailState());

    // Power Fail event is processed by application
    if (bPowerFailAppState_l)
    {
       bPowerFailAppState_l = 0;
       // re-enable power fail interrupt
       PowerFailEnableObservance();
    }

    //----------------------------------------------------------------------
    // Power Fail Stop-Mode
    bRunState_l = 1;
    bPowerFailMonitor_l  = 0;
    bPowerFailAppState_l = 0;
    
    PowerFailStartCriticalSection();

    while (bRunState_l);

    PowerFailEndCriticalSection();

    // Application stops in Callback handler
    while (1);
}

//-------------------------------------------------------------------------
// function replacement for missing Watchdog driver
void C14EnableWatchdog(void)
{
}

//-------------------------------------------------------------------------
// function replacement for missing Watchdog driver
void C14ResetWatchdog (void)
{
}
