/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Command interpreter

  Description:  Sourcen of the command interpreter

  -------------------------------------------------------------------------

                $RCSfile: CmdInt.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2004/01/12 -ct:   - convert all comments into english language
                    - adapt to generic SIO-Driver


****************************************************************************/

#include <stdlib.h>
#include <string.h>

#include "generic/global.h"
#include "../include/cmdint.h"

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

// Include Prototypes for hardwarespezifische Funktionen (File CmdIntTgt.c)
void PUBLIC CmtIntTgtInitSioDrv (void);
char PUBLIC CmdIntTgtPutChar    (char cZeichen);
BYTE PUBLIC CmdIntTgtGetChar    (void);


/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <CmdTerminal>                                       */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
// Description:
// Verarbeiten von Cursor-Zeichen:
// Der Tastaturcode fr die Cursor-Tasten besteht aus drei Zeichen (Pfeil
// aufwrts = 0x1B 0x5B 0x41).
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define EOS             0x00            // End Of String                   
#define BS              0x08            // BackSpace                       
#define LF              0x0A            // Line Feed                       
#define CR              0x0D            // Carriage Return                 
#define ESC             0x1B            // Escape                          
#define DC1             0x11            // Control character for KEIL-Monitor

#define NO_ARGUMENT     5               // Number of arguments
#define MAX_STRING_SIZE 50              // max. size of string

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------


//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

static char            cCommand_l[80];     // Command-Buffer
static WORD            wCmdIdx_l;          // index to Command[]

static tCmdTableEntry* pCmdTable_l;        // Command table
static WORD            wCmdEntries_l;      // number of entries in command table

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

static BYTE CmdIntGetCommand     (void);
static void CmdIntPrintCrLf      (void);
static void CmdIntPrintReady     (void);

static void strupr               (char *pString_p);

static char ** CmdIntGetArg      (char * pcCommandLine_p);
static void CmdInterpreteCommand (char ** pArgList_p);

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     CmdIntInit
//
// Description:  This function initialize the command interpreter.
//
// Parameters:   pointer to the command table
//               count of index in the table
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC CmdIntInit (tCmdTableEntry* pCmdTable_p, WORD wCmdEntries_p)
{

    // register the command table
    pCmdTable_l   = pCmdTable_p;
    wCmdEntries_l = wCmdEntries_p;

    // clear the command buffer
    wCmdIdx_l = 0;

    // get pointer of the SioDrv instance
    CmtIntTgtInitSioDrv();

    CmdIntPrintStr("\r\nStarting command interpreter ...");
    CmdIntPrintStr("\r\n> (type 'help' for command list)");
    CmdIntPrintReady();
}


//---------------------------------------------------------------------------
//
// Function:     CmdIntProcess
//
// Description:  This function checks if a command is received. If one is
//               received, this command is processed
//
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC CmdIntProcess (void)
{
char** pArg;

    /*-------------------- command working ---------------------------*/
    if (CmdIntGetCommand() )   // receive a command ?
    {                                   
       // check for the argument list
       pArg = CmdIntGetArg  (cCommand_l);

       // process command in the application
       CmdInterpreteCommand (pArg);

       CmdIntPrintReady();
    }
}

//---------------------------------------------------------------------------
//
// Function:     CmdInitCommand
//
// Description:  This function calls the init-function for one command class.
//
//
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC CmdInitCommand (void)
{
WORD wIndx;

    // search for the command in the table
    for (wIndx=0; wIndx < wCmdEntries_l; wIndx++)
    {
        // is a pointer for the initfunction defined ?     
        if (pCmdTable_l[wIndx].m_fpCmdInit)
           pCmdTable_l[wIndx].m_fpCmdInit();
    }
}

//---------------------------------------------------------------------------
//
// Function:     CmdPrintHelp
//
// Description:  This function prints the help information of the commands
//
// Parameters:   none
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

void PUBLIC CmdPrintHelp (void)
{
WORD wIndx;

    CmdIntPrintCrLf();

    for (wIndx=0; wIndx < wCmdEntries_l; wIndx++)
    {
        // print out the command
        CmdIntPrintStr(pCmdTable_l[wIndx].m_pstrCommand);

        // print out the help wording
        CmdIntPrintStr(pCmdTable_l[wIndx].m_pstrCommandDescription);

    }
}

//---------------------------------------------------------------------------
// Function:     CmdIntPrintStr
//
// Description:  Function sends out an string.
//
// Parameters:   pointer to string
// 
// Returns:      none
//
// State:        
//---------------------------------------------------------------------------

void PUBLIC CmdIntPrintStr(BYTE FAR * pbString_p)
{
    printf("%s", pbString_p);
}


//=========================================================================//
//                                                                         //
//          P R I V A T E   F U N C T I O N S                              //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     CmdIntGetCommand
//
// Description:  This function collects the character of the ASC0 interface
//               into the receive buffer. If a CR is received, the command 
//               is complete and has to process.
//
// Parameters:   none
// 
// Returns:      TRUE   ===> save command in the buffer
//               FALSE  ===> no complete command detect
//
// State:        
//
//---------------------------------------------------------------------------

static BYTE CmdIntGetCommand (void)
{
BYTE bChr = 0x00;
BYTE bErr = FALSE;

    // read out a character of the ASC0
    bChr = CmdIntTgtGetChar();
                                       
    // received a char
    if (bChr != 0x00)
    {
       //--------------------------------------------------------------
       // Zeichenverarbeitung
       switch (bChr)
       {
          //--------------------------------------------------------------
          case DC1: // control char of the debugger  ==> do not process

          break;

          //--------------------------------------------------------------
          case CR:  // end of command
               CmdIntTgtPutChar(CR);        // send echo
               cCommand_l[wCmdIdx_l] = EOS; // write end of string
               strupr( cCommand_l );        // convert into capital letter
               if (wCmdIdx_l)               // char contain in Command[] ?
               {                       
                    wCmdIdx_l = 0;            // set index to the start of command[] 
                    bErr = TRUE;              // end with "command ready"
               }
               else
               {
                    // ersten Zeichen war CR -> erneut Bereitschaft zeigen
                    CmdIntPrintReady();
               }
          break;

          //--------------------------------------------------------------
          case BS:  // BackSpace
               // already get characters ?
               if (wCmdIdx_l > 0)                  
               {
                  CmdIntTgtPutChar(BS);
                  CmdIntTgtPutChar(' ');
                  CmdIntTgtPutChar(BS);

                  wCmdIdx_l--;              // clear char in command[]
               }
          break;

          //--------------------------------------------------------------
          case ESC:  // Escape ?
               // already get characters ?
               if (wCmdIdx_l == 0)
               { 
                  bErr = TRUE;              // last command again
               }
          break;
         
          //--------------------------------------------------------------
          default:  
               // One normal char is received.
               // Es werden nur Zeichen abgelegt, wenn im Puffer
               // noch Platz ist. Es mu jedoch mglich sein, da
               // stets noch EOS abgelegt werden kann. Anderenfalls
               // kann sonst nie die Queue wieder geleert werden.
               if (wCmdIdx_l < (tab_entries(cCommand_l) - 1))
               {
                  CmdIntTgtPutChar(bChr);
                  cCommand_l[wCmdIdx_l++] = bChr; // save character
               }
          break;
       }
    }

    return(bErr);
}

//---------------------------------------------------------------------------
//
// Function:     CmdIntGetArg
//
// Description:  This function gets the pointer at the list of argument
//               of one command. It can find out for max 5 arguments.
//
// Parameters:   pointer to the command
//
// Returns:      pointer to the list of argument
//               
// State:        
//
//---------------------------------------------------------------------------

static char ** CmdIntGetArg (char * pcCommandLine_p)
{
WORD         wIndx = 0;
char*        pCommandLine = pcCommandLine_p;
static char* pArgList[1+NO_ARGUMENT+1];

      // ignore leading spaces
      while (*pCommandLine == ' ')
      {
            pCommandLine++;
      }

      // save the argument pointer
      pArgList[wIndx++] = pCommandLine;
      
      // already get characters ?
      while ( *pCommandLine && (wIndx < (tab_entries(pArgList) - 1)) )
      {
          // search for the next space
          if (*pCommandLine != ' ')
          {
             pCommandLine++;
             continue;
          }

          // find space, ignore the next spaces
          while (*pCommandLine == ' ') 
          {
                pCommandLine++;
          }

          // save the pointer to the argument
          pArgList[wIndx++] = pCommandLine;
      }

      pArgList[wIndx] = (char*)NULL;

    return (pArgList);
}

//---------------------------------------------------------------------------
//
// Function:     CmdInterpreteCommand
//
// Description:  This function searchs for a received command the call-back
//               function and called this 
//
// Parameters:   pointer at the received command
//
// Returns:      none
//
// State:        
//
//---------------------------------------------------------------------------

static void CmdInterpreteCommand (char ** pArgList_p)
{
WORD wIndx;

    if (!strcmp(pArgList_p[0],"HELP"))  // help command ?
    {
       CmdPrintHelp();
       return;
    }

    // search command in the comnmand table 
    for (wIndx=0; wIndx < wCmdEntries_l; wIndx++)
    {
        // find command ?     
        if ( !memcmp (pArgList_p[0],
                      pCmdTable_l[wIndx].m_pstrCommand, pCmdTable_l[wIndx].m_wCmdSize))
        {
           // has the command valid parameter?
           if (pCmdTable_l[wIndx].m_fpCmdProcess)
           {  // call the command function and get out of the interpreter
              pCmdTable_l[wIndx].m_fpCmdProcess(pArgList_p);
              goto EXIT;
           }
        }
    }

    // no valid command is received
    CmdIntPrintStr("\nOops?!\n");

EXIT:
    return;
}

//---------------------------------------------------------------------------
// Function:         CmdIntPrintCrLf
//
// Description:  
//
// Parameters:   
// 
// Returns:      
//
// State:        
//---------------------------------------------------------------------------
static void CmdIntPrintCrLf (void)
{
    CmdIntTgtPutChar(CR);
    CmdIntTgtPutChar(LF);
}

//---------------------------------------------------------------------------
// Function:      CmdIntPrintReady
//
// Description:  
//
// Parameters:   
// 
// Returns:      
//
// State:        
//---------------------------------------------------------------------------

static void CmdIntPrintReady (void)
{
    CmdIntPrintCrLf();
    CmdIntTgtPutChar('>');
    CmdIntTgtPutChar(' ');
}

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <character type conversion>                         */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/


//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// lokale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

//=========================================================================//
//                                                                         //
//          P U B L I C   F U N C T I O N S                                //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
//
// Function:     htoul
//
// Description:  Diese Funktion  wandelt einen ASCII-HEX-String in
//               einen 'DWORD'-Wert um. Der String mu mit einem  
//               von  den Ziffern '0' bis 'F' verschiedenen Zeichen 
//               abgeschlossen  sein. Fhrende Nullen werden unterdrckt
//
//               Es ist  darauf zu achten, da  die im String enthaltene  
//               Unsigned-Long-Zahl den Wertebereich  fr Unsigned-Long 
//               nicht berschreitet!          
//
// Parameters:   Pointer auf den zu wandelnden String
//
// 
// Returns:      DWORD-Wert
//
//
// State:        
//
//---------------------------------------------------------------------------

DWORD PUBLIC htoul (char *str)
{
DWORD uval;
int   nib;

    if( *str == '\0' ) 
      return( 0 );
    
    uval = 0;
    while( (*str>='0' && *str<='9') ||
         (*str>='a' && *str<='f')   ||
         (*str>='A' && *str<='F')  )
    {
        if( *str >= 'a' )
        {
          nib = *str  - 'a' + 0xA;
        }
        else
        {
            if( *str >= 'A' )
            {
              nib = *str  - 'A' + 0xA;
            }
            else
            {
              nib = *str  - '0';
            }
        }
        uval <<= 4;
        uval += (DWORD)nib;
        str++;
    }
    
    return( uval );
}

//---------------------------------------------------------------------------
//
// Function:     strupr
//
// Description:  This function convert a string into capital letters 
//
// Parameters:   pointer at the string
//
// Returns:      keine
//
// State:        
//
//---------------------------------------------------------------------------

static void strupr(char *pString_p)
{
    // get the end of the string ?
    while (*pString_p)
    {
        if ( (*pString_p>='a') && (*pString_p<='z') )
        {
           *pString_p &= (BYTE)0xDF;
        }

        pString_p++;
    }
}


// EOF








