/****************************************************************************

  (c) SYSTEC electronic GmbH, D-07973 Greiz, August-Bebel-Str. 29
      www.systec-electronic.de

  Project:      Generic / Project independent

  Description:  Trap monitor
                
                This traphandler prints a message when a trap 0x4(Stack 
                overflow), 0x6(Stack underflow) and 0xa(Class B) occures.
                It prints the register contents for class B hardware traps. 
                The printed registers are read from the stack. It depends 
                on the compiler which registers are saved, therefore.
                
                The array regist[] is correct for Keil-C166.exe V4.06. 
                It's maybe different on other versions.

  -------------------------------------------------------------------------

                $RCSfile: TrapMon.c,v $
                
                $Author: A.Suess $
                
                $Revision: 1.1.1.1 $  $Date: 2004/09/16 11:56:05 $
                
                $State: Exp $
                
                Build Environment:
                KEIL uVision 2

  -------------------------------------------------------------------------

  Revision History:

  2003/02/12 -as:   Start der Implementierung

****************************************************************************/
 
#ifdef __C166__
    #include <xc161.h>
    #include <intrins.h>
#endif

#include <stdio.h>
#include "generic/global.h"





/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          G L O B A L   D E F I N I T I O N S                            */
/*                                                                         */
/*                                                                         */
/***************************************************************************/

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

#define _ENABLE_SHOW_TRAP_ON_LED_


#ifdef _ENABLE_SHOW_TRAP_ON_LED_

    // Konfiguration fuer C14-CPU
    sbit DirErrLED = DP3^3;
    sbit PinErrLED = P3^3;

    sbit DirRunLED = DP3^4;
    sbit PinRunLED = P3^4;


    #define SHOW_TRAP_ON_LED()  {DirErrLED=1; PinErrLED=0; DirRunLED=1; PinRunLED=0;}

#else

    #define SHOW_TRAP_ON_LED()

#endif



//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local function prototypes
//---------------------------------------------------------------------------

/***************************************************************************/
/*                                                                         */
/*                                                                         */
/*          C L A S S  <TrapMon>                                           */
/*                                                                         */
/*                                                                         */
/***************************************************************************/
//
//  Description:
//
//
/***************************************************************************/

//=========================================================================//
//                                                                         //
//          P R I V A T E   D E F I N I T I O N S                          //
//                                                                         //
//=========================================================================//

//---------------------------------------------------------------------------
// const defines
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// local types
//---------------------------------------------------------------------------

//---------------------------------------------------------------------------
// modul globale vars
//---------------------------------------------------------------------------
const char *const regist[] = {
        "R13=", "R12=", "R11=", "R10=", "R9 =", "R8 =", "R7 =", "R6 =",
        "R5 =", "R4 =", "R3 =", "R2 =", "R1 =", "MDL=", "MDH=", "MDC=",
        "DP0=", "DP3=", "IP =", "CSP=", "PSW=" };

//---------------------------------------------------------------------------
//
// Function:     TRAPHandlerNMI
//
// Description:  Trap Handler for NMI (cannot be used in monitor mode)
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

/*
static void  TRAPHandlerNMI (void) interrupt TRAP=0x02
{
    TFR = 0;    // NMI-Bit im Trap Flag Register lschen
}
*/

//---------------------------------------------------------------------------
//
// Function:     TRAPHandlerSTKOF
//
// Description:  Trap Handler for stack overflow.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TRAPHandlerSTKOF (void)interrupt TRAP=0x04
{

static unsigned int wTFR;


    SHOW_TRAP_ON_LED();

    wTFR = TFR;
    TFR  = 0;  // reset Trap Status Flag Register
    
    // Stack neu initialisieren, um die weiteren Funktionen 
    // ausfhren zu knnen
    SP = 0xFC00;
    _nop_();
    
    TRACE0("\nTrap Stack Overflow");
    
    // Knoten zurcksetzen
    _trap_(0);
    
    while (1);

}



//---------------------------------------------------------------------------
//
// Function:     TRAPHandlerSTKUF
//
// Description:  Trap Handler for stack underflow.
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TRAPHandlerSTKUF (void)interrupt TRAP=0x06
{

static unsigned int wTFR;


    SHOW_TRAP_ON_LED();

    wTFR = TFR;
    TFR  = 0;  // reset Trap Status Flag Register
    
    // Stack neu initialisieren, um die weiteren Funktionen 
    // ausfhren zu knnen
    SP = 0xFC00;
    _nop_();
    
    TRACE0("\nTrap Stack Underflow");
    
    // Knoten zurcksetzen
    _trap_(0x00);
    
    while (1);

}



//---------------------------------------------------------------------------
//
// Function:     TRAPHandlerClassB
//
// Description:  Trap Handler for CLASS B hardware traps.
//
//               - Undefined Opcode
//               - Protected Instruction fault
//               - Illegal Word Operand Access
//               - Illegal Instruction Access
//               - Illegal External Bus Access
//
//
// Parameters:   keine
//
// 
// Returns:      keine
//
//
// State:        
//
//---------------------------------------------------------------------------

static void TRAPHandlerClassB (void)interrupt TRAP=0x0A
{

static unsigned int wTFR;
static unsigned int wSP;
static unsigned int awStack[0x40];
static unsigned int w;


    SHOW_TRAP_ON_LED();

    wTFR = TFR;
    wSP = SP;

    TFR  = 0;  // reset Trap Status Flag Register

    for( w=0; w<sizeof(awStack)/sizeof(*awStack); w++)
    {
        awStack[w] = *((unsigned int huge *)wSP + w);
    }
    
    // Stack neu initialisieren, um die weiteren Funktionen 
    // ausfhren zu knnen
    SP = 0xFC00;
    _nop_();
    
    TRACE1("\nTrap Class B: TFR=0x%04X \n", wTFR);

    if ( wTFR & 0x0001 ) 
    {
       TRACE0("  Illegal External Bus Access\n");
    }

    if ( wTFR & 0x0002 ) 
    {
       TRACE0("  Illegal Instruction Access\n");
    }

    if ( wTFR & 0x0004 ) 
    {
       TRACE0("  Illegal Word Operand Access\n");
    }

    if ( wTFR & 0x0008 ) 
    {
       TRACE0("  Protection Fault\n");
    }

    if ( wTFR & 0x0080 ) 
    {
       TRACE0("  Undefined Opcode\n");
    }

    TRACE1("\nStackpointer: 0x%08p\n", wSP);

    TRACE0("\nStackcontent:\n  ");
    
    for( w=0; w<sizeof(awStack)/sizeof(*awStack); w++)
    {
        TRACE2(" %s0x%04x", w<sizeof(regist)/sizeof(*regist) ? regist[w]:"    ", awStack[w] );

        if( w%8 == 7 )
        {
            TRACE0("\n  ");
        }
    }

    TRACE0("\n");


    while (1);

}



// EOF


